@@ -176,7 +176,7 @@ static inline const char *_basename(const char *str) {
 	return name;
 }
 
-void lvm_fs_create_lv(vg_t *vg_ptr, lv_t *lv) {
+devfs_handle_t lvm_fs_create_lv(vg_t *vg_ptr, lv_t *lv) {
 	struct proc_dir_entry *pde;
 	const char *name = _basename(lv->lv_name);
 
@@ -193,6 +193,11 @@ void lvm_fs_create_lv(vg_t *vg_ptr, lv_t *lv) {
 		pde->read_proc = _proc_read_lv;
 		pde->data = lv;
 	}
+#if LINUX_VERSION_CODE > KERNEL_VERSION ( 2, 3, 46)
+	return lv_devfs_handle[MINOR(lv->lv_dev)];
+#else
+	return NULL;
+#endif
 }
 
 void lvm_fs_remove_lv(vg_t *vg_ptr, lv_t *lv) {
@@ -340,8 +345,8 @@ static int _proc_read_global(char *page, char **start, off_t pos, int count,
 
 #ifdef DEBUG_LVM_PROC_GET_INFO
 	printk(KERN_DEBUG
-	       "%s - lvm_proc_get_global_info CALLED  pos: %lu  count: %d  whence: %d\n",
-	       lvm_name, pos, count, whence);
+	       "%s - lvm_proc_get_global_info CALLED  pos: %lu  count: %d\n",
+	       lvm_name, pos, count);
 #endif
 
 	if(pos != 0 && buf != NULL)
