@@ -39,6 +39,8 @@
  *                 o pv number is returned in new uint * arg
  *                 o -1 returned on error
  *                 lvm_snapshot_fill_COW_table has a return value too.
+ *    15/10/2001 - fix snapshot alignment problem [CM]
+ *               - fix snapshot full oops (always check lv_block_exception) [CM]
  *
  */
 
@@ -49,6 +51,7 @@
 #include <linux/types.h>
 #include <linux/iobuf.h>
 #include <linux/lvm.h>
+#include <linux/devfs_fs_kernel.h>
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION ( 2, 3 ,0)
 #include <linux/pagemap.h>
@@ -86,21 +89,22 @@ static inline int __brw_kiovec(int rw, int nr, struct kiobuf *iovec[],
 }
 
 
-static int _pv_get_number(vg_t * vg, kdev_t rdev, uint *pvn) {
+static int _pv_get_number(vg_t * vg, kdev_t rdev, uint *pvn)
+{
 	uint p;
-	for(p = 0; p < vg->pv_max; p++) {
-		if(vg->pv[p] == NULL)
+	for (p = 0; p < vg->pv_max; p++) {
+		if (vg->pv[p] == NULL)
 			continue;
 
-		if(vg->pv[p]->pv_dev == rdev)
+		if (vg->pv[p]->pv_dev == rdev)
 			break;
 	}
 
-	if(p >= vg->pv_max) {
+	if (p >= vg->pv_max) {
 		/* bad news, the snapshot COW table is probably corrupt */
 		printk(KERN_ERR
 		       "%s -- _pv_get_number failed for rdev = %u\n",
-		      lvm_name, rdev);
+		       lvm_name, rdev);
 		return -1;
 	}
 
@@ -153,10 +157,20 @@ inline void lvm_hash_link(lv_block_exception_t * exception,
 	unsigned long mask = lv->lv_snapshot_hash_mask;
 	int chunk_size = lv->lv_chunk_size;
 
+	if (!hash_table)
+		BUG();
 	hash_table = &hash_table[hashfn(org_dev, org_start, mask, chunk_size)];
 	list_add(&exception->hash, hash_table);
 }
 
+/*
+ * Determine if we already have a snapshot chunk for this block.
+ * Return: 1 if it the chunk already exists
+ *         0 if we need to COW this block and allocate a new chunk
+ *        -1 if the snapshot was disabled because it ran out of space
+ *
+ * We need to be holding at least a read lock on lv->lv_lock.
+ */
 int lvm_snapshot_remap_block(kdev_t * org_dev, unsigned long * org_sector,
 			     unsigned long pe_start, lv_t * lv)
 {
@@ -166,6 +180,9 @@ int lvm_snapshot_remap_block(kdev_t * org_dev, unsigned long * org_sector,
 	int chunk_size = lv->lv_chunk_size;
 	lv_block_exception_t * exception;
 
+	if (!lv->lv_block_exception)
+		return -1;
+
 	pe_off = pe_start % chunk_size;
 	pe_adjustment = (*org_sector-pe_off) % chunk_size;
 	__org_start = *org_sector - pe_adjustment;
@@ -218,7 +235,7 @@ static inline int lvm_snapshot_prepare_blocks(unsigned long *blocks,
 
 	sectors_per_block = blocksize / SECTOR_SIZE;
 
-	if(start & (sectors_per_block - 1))
+	if (start & (sectors_per_block - 1))
 		return 0;
 
 	nr_blocks = nr_sectors / sectors_per_block;
@@ -269,7 +286,6 @@ static inline void invalidate_snap_cache(unsigned long start, unsigned long nr,
 
 int lvm_snapshot_fill_COW_page(vg_t * vg, lv_t * lv_snap)
 {
-	uint pvn;
 	int id = 0, is = lv_snap->lv_remap_ptr;
 	ulong blksize_snap;
 	lv_COW_table_disk_t * lv_COW_table = (lv_COW_table_disk_t *)
@@ -279,25 +295,27 @@ int lvm_snapshot_fill_COW_page(vg_t * vg, lv_t * lv_snap)
 		return 0;
 
 	is--;
-        blksize_snap =
+	blksize_snap =
 		lvm_get_blksize(lv_snap->lv_block_exception[is].rdev_new);
-        is -= is % (blksize_snap / sizeof(lv_COW_table_disk_t));
+	is -= is % (blksize_snap / sizeof(lv_COW_table_disk_t));
 
 	memset(lv_COW_table, 0, blksize_snap);
 	for ( ; is < lv_snap->lv_remap_ptr; is++, id++) {
 		/* store new COW_table entry */
 		lv_block_exception_t *be = lv_snap->lv_block_exception + is;
-		if(_pv_get_number(vg, be->rdev_org, &pvn))
+		uint pvn;
+
+		if (_pv_get_number(vg, be->rdev_org, &pvn))
 			goto bad;
 
 		lv_COW_table[id].pv_org_number = cpu_to_le64(pvn);
 		lv_COW_table[id].pv_org_rsector = cpu_to_le64(be->rsector_org);
-		if(_pv_get_number(vg, be->rdev_new, &pvn))
+
+		if (_pv_get_number(vg, be->rdev_new, &pvn))
 			goto bad;
 
 		lv_COW_table[id].pv_snap_number = cpu_to_le64(pvn);
-		lv_COW_table[id].pv_snap_rsector =
-			cpu_to_le64(be->rsector_new);
+		lv_COW_table[id].pv_snap_rsector = cpu_to_le64(be->rsector_new);
 	}
 
 	return 0;
@@ -310,6 +328,8 @@ int lvm_snapshot_fill_COW_page(vg_t * vg, lv_t * lv_snap)
 
 /*
  * writes a COW exception table sector to disk (HM)
+ *
+ * We need to hold a write lock on lv_snap->lv_lock.
  */
 int lvm_write_COW_table_block(vg_t * vg, lv_t *lv_snap)
 {
@@ -328,6 +348,10 @@ int lvm_write_COW_table_block(vg_t * vg, lv_t *lv_snap)
  * if there is no exception storage space free any longer --> release snapshot.
  *
  * this routine gets called for each _first_ write to a physical chunk.
+ *
+ * We need to hold a write lock on lv_snap->lv_lock.  It is assumed that
+ * lv->lv_block_exception is non-NULL (checked by lvm_snapshot_remap_block())
+ * when this function is called.
  */
 int lvm_snapshot_COW(kdev_t org_phys_dev,
 		     unsigned long org_phys_sector,
@@ -337,6 +361,7 @@ int lvm_snapshot_COW(kdev_t org_phys_dev,
 {
 	const char * reason;
 	unsigned long org_start, snap_start, snap_phys_dev, virt_start, pe_off;
+	unsigned long phys_start;
 	int idx = lv_snap->lv_remap_ptr, chunk_size = lv_snap->lv_chunk_size;
 	struct kiobuf * iobuf;
 	unsigned long blocks[KIO_MAX_SECTORS];
@@ -371,8 +396,8 @@ int lvm_snapshot_COW(kdev_t org_phys_dev,
 
 	iobuf = lv_snap->lv_iobuf;
 
-	blksize_org = lvm_get_blksize(org_phys_dev);
-	blksize_snap = lvm_get_blksize(snap_phys_dev);
+	blksize_org = lvm_sectsize(org_phys_dev);
+	blksize_snap = lvm_sectsize(snap_phys_dev);
 	max_blksize = max(blksize_org, blksize_snap);
 	min_blksize = min(blksize_org, blksize_snap);
 	max_sectors = KIO_MAX_SECTORS * (min_blksize>>9);
@@ -380,6 +405,9 @@ int lvm_snapshot_COW(kdev_t org_phys_dev,
 	if (chunk_size % (max_blksize>>9))
 		goto fail_blksize;
 
+	/* Don't change org_start, we need it to fill in the exception table */
+	phys_start = org_start;
+
 	while (chunk_size)
 	{
 		nr_sectors = min(chunk_size, max_sectors);
@@ -387,21 +415,24 @@ int lvm_snapshot_COW(kdev_t org_phys_dev,
 
 		iobuf->length = nr_sectors << 9;
 
-		if(!lvm_snapshot_prepare_blocks(blocks, org_start,
-						nr_sectors, blksize_org))
+		if (!lvm_snapshot_prepare_blocks(blocks, phys_start,
+						 nr_sectors, blksize_org))
 			goto fail_prepare;
 
 		if (__brw_kiovec(READ, 1, &iobuf, org_phys_dev, blocks,
 				 blksize_org, lv_snap) != (nr_sectors<<9))
 			goto fail_raw_read;
 
-		if(!lvm_snapshot_prepare_blocks(blocks, snap_start,
-						nr_sectors, blksize_snap))
+		if (!lvm_snapshot_prepare_blocks(blocks, snap_start,
+						 nr_sectors, blksize_snap))
 			goto fail_prepare;
 
 		if (__brw_kiovec(WRITE, 1, &iobuf, snap_phys_dev, blocks,
 				 blksize_snap, lv_snap) != (nr_sectors<<9))
 			goto fail_raw_write;
+
+		phys_start += nr_sectors;
+		snap_start += nr_sectors;
 	}
 
 #ifdef DEBUG_SNAPSHOT
@@ -554,7 +585,6 @@ int lvm_snapshot_alloc(lv_t * lv_snap)
 	ret = lvm_snapshot_alloc_hash_table(lv_snap);
 	if (ret) goto out_free_both_kiovecs;
 
-
 out:
 	return ret;
 
@@ -567,8 +597,7 @@ out_free_kiovec:
 	unmap_kiobuf(lv_snap->lv_iobuf);
 	free_kiovec(1, &lv_snap->lv_iobuf);
 	lv_snap->lv_iobuf = NULL;
-	if (lv_snap->lv_snapshot_hash_table != NULL)
-		vfree(lv_snap->lv_snapshot_hash_table);
+	vfree(lv_snap->lv_snapshot_hash_table);
 	lv_snap->lv_snapshot_hash_table = NULL;
 	goto out;
 }
@@ -629,7 +658,7 @@ static int _write_COW_table_block(vg_t *vg, lv_t *lv_snap,
 	snap_phys_dev = lv_snap->lv_block_exception[idx].rdev_new;
 	snap_pe_start = lv_snap->lv_block_exception[idx - (idx % COW_entries_per_pe)].rsector_new - lv_snap->lv_chunk_size;
 
-	blksize_snap = lvm_get_blksize(snap_phys_dev);
+	blksize_snap = lvm_sectsize(snap_phys_dev);
 
         COW_entries_per_block = blksize_snap / sizeof(lv_COW_table_disk_t);
         idx_COW_table = idx % COW_entries_per_pe % COW_entries_per_block;
@@ -678,7 +707,7 @@ static int _write_COW_table_block(vg_t *vg, lv_t *lv_snap,
 			idx++;
 			snap_phys_dev = lv_snap->lv_block_exception[idx].rdev_new;
 			snap_pe_start = lv_snap->lv_block_exception[idx - (idx % COW_entries_per_pe)].rsector_new - lv_snap->lv_chunk_size;
-			blksize_snap = lvm_get_blksize(snap_phys_dev);
+			blksize_snap = lvm_sectsize(snap_phys_dev);
 			blocks[0] = snap_pe_start >> (blksize_snap >> 10);
 		} else blocks[0]++;
 
