@@ -71,6 +71,7 @@
  *    21/06/2001 - changed BLOCK_SIZE back to 1024 for non S/390
  *    22/06/2001 - added Andreas Dilger's PE on 4k boundary alignment enhancements
  *    19/07/2001 - added rwsem compatibility macros for 2.2 kernels
+ *    13/11/2001 - reduced userspace inclusion of kernel headers to a minimum
  *
  */
 
@@ -78,8 +79,8 @@
 #ifndef _LVM_H_INCLUDE
 #define _LVM_H_INCLUDE
 
-#define LVM_RELEASE_NAME "1.0"
-#define LVM_RELEASE_DATE "14/08/2001"
+#define LVM_RELEASE_NAME "1.0.1"
+#define LVM_RELEASE_DATE "26/11/2001"
 
 #define	_LVM_KERNEL_H_VERSION	"LVM "LVM_RELEASE_NAME" ("LVM_RELEASE_DATE")"
 
@@ -109,15 +110,20 @@
 
 #include <linux/kdev_t.h>
 #include <linux/list.h>
+#include <asm/types.h>
+#include <linux/major.h>
 #else
+/* This prevents the need to include <linux/list.h> which
+   causes problems on some platforms. It's not nice but then
+   neither is the alternative. */
+struct list_head {
+        struct list_head *next, *prev;
+};
 #define __KERNEL__
 #include <linux/kdev_t.h>
-#include <linux/list.h>
 #undef __KERNEL__
 #endif				/* #ifndef __KERNEL__ */
 
-#include <asm/types.h>
-#include <linux/major.h>
 
 #ifdef __KERNEL__
 #if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 3 ,0)
@@ -161,18 +167,10 @@
 #endif
 
 /* structure version */
-#define LVM_STRUCT_VERSION 2
+#define LVM_STRUCT_VERSION 1
 
 #define	LVM_DIR_PREFIX	"/dev/"
 
-#ifndef min
-#define min(a,b) (((a)<(b))?(a):(b))
-#endif
-#ifndef max
-#define max(a,b) (((a)>(b))?(a):(b))
-#endif
-
-
 /*
  * i/o protocol version
  *
@@ -282,8 +280,9 @@
 #define	LVM_MAX_STRIPES		128	/* max # of stripes */
 #define	LVM_MAX_SIZE            ( 1024LU * 1024 / SECTOR_SIZE * 1024 * 1024)	/* 1TB[sectors] */
 #define	LVM_MAX_MIRRORS    	2	/* future use */
-#define	LVM_MIN_READ_AHEAD	2	/* minimum read ahead sectors */
-#define	LVM_MAX_READ_AHEAD	120	/* maximum read ahead sectors */
+#define	LVM_MIN_READ_AHEAD	0	/* minimum read ahead sectors */
+#define	LVM_DEFAULT_READ_AHEAD	1024	/* sectors for 512k scsi segments */
+#define	LVM_MAX_READ_AHEAD	10000	/* maximum read ahead sectors */
 #define	LVM_MAX_LV_IO_TIMEOUT	60	/* seconds I/O timeout (future use) */
 #define	LVM_PARTITION           0xfe	/* LVM partition id */
 #define	LVM_NEW_PARTITION       0x8e	/* new LVM partition id (10/09/1999) */
@@ -586,7 +585,7 @@ typedef struct lv_v5 {
 	struct wait_queue *lv_snapshot_wait;
 #endif
 	int	lv_snapshot_use_rate;
-	void	*vg;
+	struct vg_v3	*vg;
 
 	uint lv_allocated_snapshot_le;
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 3, 0)
