@@ -6,6 +6,7 @@
  * June,September 1998
  * February,July,October 1999
  * January,February 2000
+ * January,February 2001
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,10 +40,20 @@
  *    01/01/2000 - take care of rounding if striped resize
  *                 (idea by Pete Allen <petea3@attglobal.net>)
  *    15/02/2000 - use lvm_error()
+ *    23/01/2001 - added call to lvm_init (JT)
+ *    19/02/2001 - incorporated support for ext2online <adilger@turbolinux.com>
+ *               - use getmntent on /etc/mtab, then /proc/mounts for mounted LV
+ *               - also check mountpoint in addition to device name.
+ *               - check /proc/mounts first, don't run e2fsck for ext2resize
+ *               - swap bytes on ext2 superblock values for big endian systems
+ *    06/04/2001 - Tidy usage message and command-line options.
  *
  */
 
 #include <lvm_user.h>
+#include <mntent.h>
+
+#define	CMD_LEN	256
 
 char *cmd = NULL;
 
@@ -50,29 +61,53 @@ char *cmd = NULL;
 int opt_d = 0;
 #endif
 
+void usage ( int ret)
+{
+   FILE *stream = stderr;
+
+   if ( ret == 0) {
+      stream = stdout;
+      printf ( "%s (IOP %d)\n\n", lvm_version, LVM_LIB_IOP_VERSION);
+      printf ( "%s -- resize logical volume and ext2 filesystem\n", cmd);
+   }
+
+   fprintf ( stream, "\n%s "
+             DEBUG_HELP_2
+             "[-h|--help] "
+             "[-n|--nofsck]\n\t"
+             "{[-l|--extents] [+|-]LogicalExtentsNumber |\n\t"
+             " [-L|--size] [+|-]LogicalVolumeSize[kKmMgGtT]}\n\t"
+             "[-t|--test] "
+             "[-v|--verbose] "
+             "[--version] "
+             "LogicalVolumePath\n\n",
+             cmd);
+   exit ( ret);
+}
+
 typedef struct {
    uint32_t	dummy1;
-   uint32_t	blocks_count;
+   uint32_t	s_blocks_count;
    uint32_t	dummy2[4];
-   uint32_t	block_size;
+   uint32_t	s_log_block_size;
    uint32_t	dummy3[7];
    uint16_t	s_magic;
-   uint32_t	dummy4[454];
+   uint16_t	dummy4;
+   uint32_t	dummy5[241];
 } ext2_super_t;
 
-static char *bin_dirs[] = { "/sbin", "/usr/sbin", "/usr/local/sbin", "/bin", "/usr/bin", "/usr/local/bin", NULL};
+static char *bin_dirs[] = { "/sbin", "/usr/sbin", "/usr/local/sbin", "/bin", "/usr/bin", NULL};
+static char *mnt_files[] = { "/proc/mounts", MOUNTED, MNTTAB , NULL};
 
 /* internal function prototypes */
 char *lvm_find_command ( char *);
 void lvm_path_error ( char *, char *);
 ext2_super_t *lvm_is_ext2_fs ( char *);
 
-
 int main ( int argc, char **argv) {
    int c = 0;
    int grow = UNDEF;
    int l = 0;
-   int len = 0;
    int opt_l = 0;
    int opt_L = 0;
    int opt_n = 0;
@@ -85,180 +120,194 @@ int main ( int argc, char **argv) {
    int size_invalid = FALSE;
    long size = 0;
    char *cmd_fsck = "e2fsck";
+   /* FIXME check "resize.<fstype>" and "oresize.<fstype>" instead */
    char *cmd_resize = NULL;
-   char *cmd_resize_default = "resize2fs";
    char *cmd_resize_opts = NULL;
+   char *cmd_resize_default = "resize2fs";
    char *cmd_resize_opts_default = "-p";
+   char *cmd_resize_default2 = "ext2resize";
+   char *cmd_resize_opts_default2 = "-v";
+   char *cmd_online = NULL;
+   char *cmd_online_default = "ext2online";
+   char *cmd_online_opts = NULL;
+   char *cmd_online_opts_default = "-v";
    char *cmd_extend = "lvextend";
    char *cmd_ptr = NULL;
    char *cmd_reduce = "lvreduce";
    char *dummy = NULL;
    char *lv_name = NULL;
-   char *mount = NULL;
    char *vg_name = NULL;
-   char *proc_mounts = "/proc/mounts";
-   char *ptr = NULL;
+   char **mntfile = NULL;
    char *size_ptr = NULL;
-   char buffer[2*NAME_LEN] = { 0, };
-   char command[2*NAME_LEN] = { 0, };
-#ifdef DEBUG
-   char *options = "dh?l:L:ntv";
-#else
-   char *options = "h?l:L:ntv";
-#endif
+   char command[CMD_LEN];
+   char *options = "h?l:L:ntv" DEBUG_SHORT_OPTION;
    struct option long_options[] = {
-#ifdef DEBUG
-      { "debug",   no_argument,       NULL, 'd'},
-#endif
+      DEBUG_LONG_OPTION
       { "help",    no_argument,       NULL, 'h'},
       { "extents", required_argument, NULL, 'l'},
       { "size",    required_argument, NULL, 'L'},
       { "nofsck",  no_argument,       NULL, 'n'},
       { "test",    no_argument,       NULL, 't'},
       { "verbose", no_argument,       NULL, 'v'},
+      { "version", no_argument,       NULL, 22 },
       { NULL,      0,                 NULL, 0}
    };
-   FILE *proc = NULL;
    lv_t *lv = NULL;
    vg_t *vg = NULL;
    ext2_super_t *super = NULL;
 
-
+   /* lvm_init ( argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
    while ( ( c = getopt_long ( argc, argv, options,
                                long_options, NULL)) != EOF) {
       switch ( c) {
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'h':
-         case '?':
-            printf ( "\n%s (IOP %d)\n\n%s -- ext2 Filesystem and LVM Administration\n\n"
-                     "Synopsis:\n"
-                     "---------\n\n"
-                     "%s\n"
-#ifdef DEBUG
-                     "\t[-d/--debug]\n"
-#endif
-                     "\t[-h/-?/--debug]\n"
-                     "\t{ -l/--extents [+/-]LogicalExtentsNumber |\n"
-                     "\t  -L/--size [+/-]LogicalVolumeSize[kKmMgGtT]}\n"
-                     "\t[-n/--nofsck]\n"
-                     "\t[-t/--test]\n"
-                     "\t[-v/--verbose]\n"
-                     "\tLogicalVolumePath\n\n",
-                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
-            return 0;
-            break;
+            usage ( 0);
 
          case 'l':
-            if ( opt_l > 0 || opt_L > 0) {
-               fprintf ( stderr, "%s -- %s option already given\n\n",
-                                 opt_l > 0 ? "l" : "L", cmd);
-               return LVM_EINVALID_CMD_LINE;
+            if ( opt_L) {
+               fprintf ( stderr, "%s -- l and L options incompatible\n", cmd);
+               usage ( LVM_EINVALID_CMD_LINE);
             }
             size_ptr = optarg;
             opt_l++;
             break;
 
          case 'L':
-            if ( opt_l > 0 || opt_L > 0) {
-               fprintf ( stderr, "%s -- %s option already given\n\n",
-                                 opt_l > 0 ? "l" : "L", cmd);
-               return LVM_EINVALID_CMD_LINE;
+            if ( opt_l) {
+               fprintf ( stderr, "%s -- l and L options incompatible\n", cmd);
+               usage ( LVM_EINVALID_CMD_LINE);
             }
             size_ptr = optarg;
             opt_L++;
             break;
 
          case 'n':
-            if ( opt_n > 0) {
-               fprintf ( stderr, "%s -- n option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             opt_n++;
             break;
 
          case 't':
-            if ( opt_t > 0) {
-               fprintf ( stderr, "%s -- t option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             opt_t++;
             break;
 
          case 'v':
-            if ( opt_v > 0) {
-               fprintf ( stderr, "%s -- v option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
             opt_v++;
             break;
 
+         case 22:
+	    printf ("%s: %s (IOP %d)\n", cmd, lvm_version, LVM_LIB_IOP_VERSION);
+            return 0;
+
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                       cmd, c);
-            return LVM_EINVALID_CMD_LINE;
+         case '?':
+            usage ( LVM_EINVALID_CMD_LINE);
       }
    }
 
+   /* not until all commands are converted
    CMD_MINUS_CHK;
-
-   if ( ( cmd_resize = getenv ( "E2FSADM_RESIZE_CMD" )) == NULL)
-      cmd_resize = cmd_resize_default;
-   if ( ( cmd_resize_opts = getenv ( "E2FSADM_RESIZE_OPTS" )) == NULL)
-      cmd_resize_opts = cmd_resize_opts_default;
-
-   if ( lvm_find_command ( cmd_resize) == NULL) {
-      lvm_path_error ( cmd_resize, "resize");
-      return LVM_EE2FSADM_RESIZE_PATH;
+   */
+   if ( optind < argc && *argv[optind] == '-') {
+      fprintf ( stderr, "%s -- invalid argument '%s'\n", cmd, argv[optind]);
+      usage ( LVM_EINVALID_CMD_LINE);
    }
 
+   if ( ( cmd_resize = getenv ( "E2FSADM_RESIZE_CMD" )) == NULL) {
+      if ( lvm_find_command ( cmd_resize_default)) {
+         cmd_resize = cmd_resize_default;
+         cmd_resize_opts = cmd_resize_opts_default;
+      } else if ( lvm_find_command ( cmd_resize_default2)) {
+         cmd_resize = cmd_resize_default2;
+         cmd_resize_opts = cmd_resize_opts_default2;
+      }
+   }
+
+
+   if ( ( dummy = getenv ( "E2FSADM_RESIZE_OPTS" )))
+      cmd_resize_opts = dummy;
+
+   if ( ( cmd_online = getenv ( "E2FSADM_ONLINE_CMD" )) == NULL) {
+      cmd_online = cmd_online_default;
+      cmd_online_opts = cmd_online_opts_default;
+   }
+
+   if ( ( dummy = getenv ( "E2FSADM_ONLINE_OPTS" )))
+      cmd_online_opts = dummy;
+
    if ( opt_l == 0 && opt_L == 0) {
-      fprintf ( stderr, "%s -- please give either an l or an L option\n\n",
-                        cmd);
-      return LVM_EINVALID_CMD_LINE;
+      fprintf ( stderr, "%s -- please give one of l or L options\n", cmd);
+      usage ( LVM_EINVALID_CMD_LINE);
    }
 
    if ( optind != argc - 1) {
-      fprintf ( stderr, "%s -- please give a logical volume name\n\n",
-                        cmd);
-      return LVM_EE2FSADM_LV_MISSING;
+      fprintf ( stderr, "%s -- please give a logical volume name\n\n", cmd);
+      usage ( LVM_EE2FSADM_LV_MISSING);
    }
 
+   if ( opt_t > 0)
+      printf ( "%s -- this is a test run and not a real resize\n", cmd);
+
    lv_name = argv[optind];
 
-   if ( opt_t > 0) printf ( "%s -- this is a test run and no real resize\n",
-                            cmd);
+   /* check to see if the LV or mountpoint appears in one of the mtab files */
+   for ( mntfile = mnt_files; *mntfile != NULL; mntfile++) {
+      struct mntent *mntent;
+      FILE *mounts;
+
+      if ( ( mounts = setmntent ( *mntfile, "r")) == NULL)
+         continue;
+
+      while ( ( mntent = getmntent ( mounts)) != NULL) {
+         /* Also check the mount point, and update lv_name if it matches */
+         if ( strcmp( lv_name, mntent->mnt_dir) == 0)
+            lv_name = mntent->mnt_fsname;
+
+         if ( strcmp ( lv_name, mntent->mnt_fsname) == 0) {
+            /* If we aren't looking at /etc/fstab, the filesystem is mounted */
+            if ( strcmp ( *mntfile, MNTTAB)) {
+               if ( lvm_find_command ( cmd_online) == NULL) {
+                  lvm_path_error ( cmd_online, "online resize");
+                  fprintf ( stderr, "%s -- ERROR: %s says %s is mounted on "
+                                    "%s.\nPlease unmount it to resize.\n\n",
+                                    cmd, *mntfile, lv_name, mntent->mnt_dir);
+                  endmntent ( mounts);
+                  return LVM_EE2FSADM_RESIZE_PATH;
+               }
+               cmd_resize = cmd_online;
+               cmd_resize_opts = cmd_online_opts;
+            }
+            /* don't close mounts, in case we point to mnt_fsname */
+            goto have_mntent;
+         }
+      }
+      endmntent ( mounts);
+   }
+have_mntent:
 
    if ( opt_v > 0) printf ( "%s -- checking logical volume name\n", cmd);
    if ( lv_check_name ( lv_name) < 0) {
       fprintf ( stderr, "%s -- invalid logical volume name \"%s\"\n\n",
                         cmd, lv_name);
-      return LVM_EE2FSADM_LVNAME;
+      usage ( LVM_EE2FSADM_LVNAME);
    }
 
-   if ( opt_v > 0) printf ( "%s -- checking logical volume \"%s\" existance\n",
-                            cmd, lv_name);
+   if ( opt_v > 0)
+      printf ( "%s -- checking existance of logical volume %s\n", cmd, lv_name);
    if ( lvm_tab_lv_check_exist ( lv_name) != TRUE) {
       fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n\n",
                         cmd, lv_name);
-      return LVM_EE2FSADM_LV_EXIST;
+      usage ( LVM_EE2FSADM_LV_EXIST);
    }
 
-   if ( opt_v > 0) printf ( "%s -- checking for relative/absolute "
-                            "size change\n", cmd);
+   if ( opt_v > 0)
+      printf ( "%s -- checking for relative/absolute size change\n", cmd);
 
    if ( *size_ptr == '+') {
       sign = +1;
@@ -269,9 +318,9 @@ int main ( int argc, char **argv) {
    }
 
    if ( ( size = lvm_check_number ( size_ptr, opt_L > 0 ? TRUE : FALSE)) < 0) {
-      fprintf ( stderr, "%s -- invalid filesystem size %s\"%s\"\n\n",
-                        cmd, sign > 0 ? "+" : "-", size_ptr);
-      return LVM_EE2FSADM_FSSIZE;
+      fprintf ( stderr, "%s -- invalid filesystem size %c%s\n", cmd,
+                sign > 0 ? '+' : '-', size_ptr);
+      usage ( LVM_EE2FSADM_FSSIZE);
    }
 
    vg_name = vg_name_of_lv ( lv_name);
@@ -318,9 +367,8 @@ int main ( int argc, char **argv) {
    }
 
    if ( size_invalid == TRUE) {
-      fprintf ( stderr, "%s -- size of \"%s\" would%s\n\n",
-                        cmd, lv_name,
-                        size_equal == TRUE ? "n't change" : " be invalid");
+      fprintf ( stderr, "%s -- size of \"%s\" would %s\n\n", cmd, lv_name,
+                size_equal == TRUE ? "not change" : " be invalid");
       return LVM_EE2FSADM_FSSIZE_CHANGE;
    }
 
@@ -343,48 +391,26 @@ int main ( int argc, char **argv) {
       printf ( "%ld KB\n", size / 2);
    }
 
-   if ( ( proc = fopen ( proc_mounts, "r")) == NULL) {
-      fprintf ( stderr, "%s -- ERROR: can't open \"%s\"\n\n",
-                        cmd, proc_mounts);
-      return LVM_EE2FSADM_PROC_MOUNTS;
-   }
-
-   len = strlen ( lv_name);
-   while ( fgets ( buffer, sizeof ( buffer), proc) != NULL) {
-      if ( strncmp ( lv_name, buffer, len) == 0) {
-         mount = buffer;
-         while ( *mount != ' ') mount++;
-         mount++;
-         ptr = mount;
-         while ( *ptr != ' ') ptr++;
-         *ptr = 0;
-         fprintf ( stderr, "%s -- ERROR: \"%s\" is mounted on \"%s\"\n"
-                           "%s -- please umount it to resize\n\n",
-                           cmd, lv_name, mount, cmd);
-         fclose ( proc);
-         return LVM_EE2FSADM_MOUNTED;
-      }
-   }
-   fclose ( proc);
-
    if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
       return LVM_EE2FSADM_NO_EXT2;
 
-   if ( ( super->blocks_count<<super->block_size) + sign * size / 2 < 0) {
-      fprintf ( stderr, "%s -- logical volume size for \"%s\" invalid\n\n",
-                        cmd, lv_name);
+   if ( ( super->s_blocks_count << ( super->s_log_block_size+1))
+        + sign * size < 0) {
+      fprintf ( stderr,"%s -- logical volume size for \"%s\" invalid\n\n",
+                       cmd, lv_name);
       return LVM_EE2FSADM_LV_SIZE;
    }
 
-   if ( opt_n == 0) {
-      if ( opt_v > 0) printf ( "%s -- checking filesystem (this can "
-                               "take a while)\n", cmd);
+   if ( opt_n == 0 && cmd_resize != cmd_online &&
+        cmd_resize != cmd_resize_default2) {
+      if ( opt_v > 0)
+         printf ( "%s -- checking filesystem (this can take a while)\n", cmd);
       if ( ( cmd_ptr = lvm_find_command ( cmd_fsck)) == NULL) {
          lvm_path_error ( cmd_fsck, "do filesystem check");
          return LVM_EE2FSADM_FSCK_PATH;
       }
       if ( opt_t == 0) {
-         sprintf ( command, "%s -f %s%c", cmd_ptr, lv_name, 0);
+         sprintf ( command, "%s -f %s", cmd_ptr, lv_name);
          if ( system ( command) != 0) return  LVM_EE2FSADM_FSCK_RUN;
       }
    }
@@ -396,37 +422,37 @@ int main ( int argc, char **argv) {
          lvm_path_error ( cmd_extend, "extend logical volume");
          return LVM_EE2FSADM_LV_EXTEND_PATH;
       }
-      sprintf ( command, "%s -l %lu %s%c",
-                         cmd_ptr, size / vg->pe_size, lv_name, 0);
+      sprintf ( command, "%s -l %lu %s",
+                         cmd_ptr, size / vg->pe_size, lv_name);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_LV_EXTEND_RUN;
          if ( ( ret = lvm_tab_lv_read_by_name ( vg_name, lv_name, &lv)) < 0) {
-            fprintf ( stderr, "%s -- ERROR reading VGDA part of \"%s\" "
-                              "from lvmtab\n\n",
-                              cmd, lv_name);
+            fprintf ( stderr, "%s -- ERROR reading VGDA of %s from lvmtab\n\n",
+                      cmd, lv_name);
             return LVM_EE2FSADM_LV_READ;
          }
          if ( size != lv->lv_size) {
-            fprintf ( stderr, "%s -- logical volume size of \"%s\" changed "
-                              "invalid\n\n",
-                              cmd, lv_name);
+            fprintf ( stderr, "%s -- new logical volume size of %s invalid\n\n",
+                      cmd, lv_name);
             return LVM_EE2FSADM_LV_SIZE;
          }
       }
-      if ( opt_v > 0) printf ( "%s -- resizing filesystem (this can "
-                               "take a while)\n", cmd);
-      if ( ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
-         lvm_path_error ( cmd_resize, "resize");
-         return LVM_EE2FSADM_RESIZE_PATH;
+      if ( opt_v > 0)
+         printf ( "%s -- resizing filesystem (this can take a while)\n", cmd);
+      if ( cmd_resize == NULL ||
+           ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
+        lvm_path_error ( cmd_resize ?
+                         cmd_resize : cmd_resize_default, "resize");
+        return LVM_EE2FSADM_RESIZE_PATH;
       }
-      sprintf ( command, "%s %s %s %lu%c",
+      sprintf ( command, "%s %s %s %lu",
                          cmd_ptr, cmd_resize_opts,
-                         lv_name, ( size / 2)>>super->block_size, 0);
+                         lv_name, size >> ( super->s_log_block_size + 1));
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_RESIZE_RUN;
          if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
             return LVM_EE2FSADM_NO_EXT2;
-         if ( ( size / 2)>>super->block_size != super->blocks_count) {
+         if ( size >> ( super->s_log_block_size + 1) != super->s_blocks_count) {
             fprintf ( stderr, "%s -- failure of of \"%s\": \"%s\" "
                               "was not successfully extended\n",
                               cmd, cmd_ptr, lv_name);
@@ -435,20 +461,22 @@ int main ( int argc, char **argv) {
       }
    /* SHRINK */
    } else { /* grow == FALSE */
-      if ( opt_v > 0) printf ( "%s -- resizing filesystem (this can "
-                               "take a while)\n", cmd);
-      if ( ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
-         lvm_path_error ( cmd_resize, "resize");
+      if ( opt_v > 0)
+         printf ( "%s -- resizing filesystem (this can take a while)\n", cmd);
+      if ( cmd_resize == NULL ||
+           ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
+         lvm_path_error ( cmd_resize ?
+                          cmd_resize : cmd_resize_default, "resize");
          return LVM_EE2FSADM_RESIZE_PATH;
       }
-      sprintf ( command, "%s %s %s %lu%c",
+      sprintf ( command, "%s %s %s %lu",
                          cmd_ptr, cmd_resize_opts,
-                         lv_name, ( size / 2)>>super->block_size, 0);
+                         lv_name, size >> ( super->s_log_block_size + 1));
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_RESIZE_RUN;
          if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
             return LVM_EE2FSADM_NO_EXT2;
-         if ( ( size / 2)>>super->block_size != super->blocks_count) {
+         if ( size >> ( super->s_log_block_size + 1) != super->s_blocks_count) {
             fprintf ( stderr, "%s -- failure of of \"%s\": \"%s\" "
                               "was not successfully reduced\n",
                               cmd, cmd_ptr, lv_name);
@@ -460,17 +488,16 @@ int main ( int argc, char **argv) {
          lvm_path_error ( cmd_reduce, "reduce logical volume");
          return LVM_EE2FSADM_LV_REDUCE_PATH;
       }
-      sprintf ( command, "%s -f -l %lu %s%c",
-                         cmd_ptr, size / vg->pe_size, lv_name, 0);
+      sprintf ( command, "%s -f -l %lu %s",
+                         cmd_ptr, size / vg->pe_size, lv_name);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_LV_REDUCE_RUN;
       }
    }
 
-   printf ( "%s -- ext2fs in logical volume \"%s\" successfully ",
-            cmd, lv_name);
-   printf ( "%s", grow == TRUE ? "extended" : "reduced");
-   printf ( " to %s\n\n", lvm_show_size ( size / 2, SHORT));
+   printf ( "%s -- ext2fs in logical volume %s successfully %s to %s\n\n",
+            cmd, lv_name, grow == TRUE ? "extended" : "reduced",
+            lvm_show_size(size / 2, SHORT));
 
    return 0;
 }
@@ -478,18 +505,22 @@ int main ( int argc, char **argv) {
 
 char *lvm_find_command ( char *cmd_to_find) {
    int i = 0;
-   static char command[NAME_LEN];
+   char *ret = NULL;
+   static char command[CMD_LEN];
    struct stat stat_b;
 
-   if ( cmd_to_find == NULL || strlen ( cmd_to_find) == 0) return NULL;
+   if ( cmd_to_find == NULL || strlen ( cmd_to_find) == 0) return ret;
 
    while ( bin_dirs[i] != NULL) {
-      sprintf ( command, "%s/%s%c", bin_dirs[i], cmd_to_find, 0);
-      if ( stat ( command, &stat_b) == 0) return command;
+      sprintf ( command, "%s/%s", bin_dirs[i], cmd_to_find);
+      if ( stat ( command, &stat_b) == 0) {
+         ret = command;
+         break;
+      }
       i++;
    }
 
-   return NULL;
+   return ret;
 }
 
 
@@ -499,8 +530,7 @@ void lvm_path_error ( char *cmd_to_find, char *comment) {
    if ( cmd_to_find == NULL || strlen ( cmd_to_find) == 0 ||
         comment     == NULL || strlen (     comment) == 0) return;
 
-   fprintf ( stderr, "%s -- Sorry: \"%s\" not found in either of",
-                     cmd, cmd_to_find);
+   fprintf(stderr, "%s -- Sorry: '%s' not found in any of", cmd, cmd_to_find);
    while ( bin_dirs[i] != NULL) fprintf ( stderr, " %s", bin_dirs[i++]);
    fprintf ( stderr, "\n%s -- ERROR: can't %s\n\n", cmd, comment);
 
@@ -521,9 +551,8 @@ ext2_super_t *lvm_is_ext2_fs ( char *device) {
    }
 
    if ( lseek ( fs, 1024, SEEK_SET) == -1) {
-      fprintf ( stderr, "%s -- ERROR \"%s\" seeking for superblock "
-                        "on \"%s\"\n\n",
-                        cmd, lvm_error ( errno), device);
+      fprintf(stderr, "%s -- ERROR \"%s\" seeking for superblock on \"%s\"\n\n",
+              cmd, lvm_error ( errno), device);
       ret = NULL;
       goto lvm_is_ext2_fs_end;
    }
@@ -535,7 +564,7 @@ ext2_super_t *lvm_is_ext2_fs ( char *device) {
       goto lvm_is_ext2_fs_end;
    }
 
-   if ( super.s_magic != EXT2_SUPER_MAGIC) {
+   if ( super.s_magic != LVM_TO_DISK16 ( EXT2_SUPER_MAGIC)) {
       fprintf ( stderr, "%s -- ERROR: \"%s\" doesn't contain an ext2 "
                         "filesystem\n\n",
                         cmd, device);
@@ -543,9 +572,12 @@ ext2_super_t *lvm_is_ext2_fs ( char *device) {
       goto lvm_is_ext2_fs_end;
    }
 
+   /* byteswap needed superblock fields */
+   super.s_blocks_count = LVM_TO_CORE32 ( super.s_blocks_count);
+   super.s_log_block_size = LVM_TO_CORE32 ( super.s_log_block_size);
+
 lvm_is_ext2_fs_end:
    if ( fs != -1) close ( fs);
 
    return ret;
 }
-
