@@ -8,6 +8,7 @@
  * January,February,July 1999
  * February,March 2000
  * March,June 2001
+ * October 2001
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -36,7 +37,9 @@
  *   05/04/2001 - changed pv_move_pes() and pv_move_pe() prototypes
  *   09/04/2001 - added prototype for pv_get_partitioned_whole
  *   26/06/2001 - fixed pv_get_size() typecast on Alpha
- *
+ *   16/01/2001 - added new lvmtab functions
+ *                lvm_tab_check_free_vg_number and lvm_tab_get_all_vg_numbers
+ *   23/10/2001 -added new lvmtab function lvm_tab_check_free_lv_numbers
  */
 
 /* lvm_lib_version "LVM 0.9 by Heinz Mauelshagen   11/11/2000\n" */
@@ -120,7 +123,7 @@ extern struct config_file *config_file;
 #define	LV_MIN_NAME_LEN		5
 #define	LV_MAX_NAME_LEN		7
 #define	MIN_PART		1
-#define	MAX_PART	 	15
+#define	MAX_PART	 	64
 
 /* some metadata on the disk need to be aligned */
 #define LVM_VGDA_ALIGN 4096UL
@@ -147,9 +150,6 @@ extern struct config_file *config_file;
 
 #define	LVM_VGDA_SIZE(pv) ((pv)->pe_on_disk.base + (pv)->pe_on_disk.size)
 
-#define	LVM_PE_DISK_OFFSET(pe, pv) \
-	((pe * pv->pe_size) + pv->data_location.base)
-
 #define	LVM_PE_ALIGN		 (65536UL / SECTOR_SIZE)
 
 
@@ -160,6 +160,21 @@ extern char *cmd;
 extern int opt_d;
 #endif
 
+/* for device classification */
+typedef enum {
+  LVM_DEVICE_TYPE_IDE = 0,
+  LVM_DEVICE_TYPE_SD,
+  LVM_DEVICE_TYPE_MD,
+  LVM_DEVICE_TYPE_LOOP,
+  LVM_DEVICE_TYPE_DASD,
+  LVM_DEVICE_TYPE_DAC960,
+  LVM_DEVICE_TYPE_NBD,
+  LVM_DEVICE_TYPE_COMPAQ_SMART2,
+  LVM_DEVICE_TYPE_COMPAQ_CISS,
+  LVM_DEVICE_TYPE_UBD,
+  LVM_DEVICE_TYPE_INVALID = -1
+} lvm_device_type_t;
+
 /* for lvm_show_size () */
 typedef	enum { SHORT, LONG} size_len_t;
 
@@ -202,7 +217,7 @@ int  vg_create ( char *, vg_t *);
 int  vg_create_dir_and_group ( vg_t *);
 int  vg_create_dir_and_group_and_nodes ( vg_t *, int);
 void vg_deactivate ( char *);
-inline int vg_extend ( char *, pv_t *, vg_t *);
+int vg_extend ( char *, pv_t *, vg_t *);
 int  vg_free ( vg_t *, int);
 char *vg_name_of_lv ( char *);
 int  vg_remove ( char *);
@@ -210,7 +225,7 @@ int  vg_rename ( char *, char *);
 int  vg_read ( char *, vg_t **);
 int  vg_read_from_pv ( char *, vg_t **);
 int  vg_read_with_pv_and_lv ( char *, vg_t **);
-inline int vg_reduce ( char *, pv_t *, vg_t *);
+int vg_reduce ( char *, pv_t *, vg_t *);
 int  vg_remove_dir_and_group_and_nodes ( char *);
 int  vg_set_extendable ( char*);
 void vg_setup_pointers_for_snapshots ( vg_t*);
@@ -260,7 +275,7 @@ int    pv_flush ( char *);
 int    pv_get_index_by_kdev_t ( vg_t *, kdev_t);
 int    pv_get_index_by_name ( vg_t *, char *);
 kdev_t pv_get_kdev_t_by_number ( vg_t *, int);
-int    pv_get_size ( char *, struct partition *);
+long pv_get_size(char *, struct partition *);
 int    pv_move_pes ( vg_t*, char*, size_t, char**, int, int,
                      long*, long*, long*, int, int);
 int    pv_move_pe ( vg_t*, char*, size_t, long, long, long, long,
@@ -305,10 +320,10 @@ int    lv_check_stripesize ( int);
 lv_t      *lv_copy_from_disk ( lv_disk_t *);
 lv_disk_t *lv_copy_to_disk ( lv_t *);
 int    lv_count_pe ( pv_t *, int);
-inline int lv_create ( vg_t *, lv_t *, char *);
+int lv_create ( vg_t *, lv_t *, char *);
 int    lv_create_name ( char *, char *, int);
 int    lv_create_node ( lv_t *);
-inline int lv_extend ( vg_t *, lv_t *, char *);
+int lv_extend ( vg_t *, lv_t *, char *);
 int    lv_get_index_by_kdev_t ( vg_t *, kdev_t);
 int    lv_get_index_by_minor ( vg_t *, int);
 int    lv_get_index_by_name ( vg_t *, char *);
@@ -323,9 +338,9 @@ int    lv_read_byindex ( char *vg_name, ulong lv_index, lv_t **lv);
 int    lv_read_COW_table ( vg_t * vg, lv_t * lv);
 int    lv_read_with_pe ( char *, char *, lv_t **);
 int    lv_read_all_lv ( char *, lv_t ***, int);
-inline int lv_reduce ( vg_t *, lv_t *, char *);
+int lv_reduce ( vg_t *, lv_t *, char *);
 int    lv_release ( vg_t *, char *);
-inline int lv_remove ( vg_t *, lv_t *, char *);
+int lv_remove ( vg_t *, lv_t *, char *);
 int    lv_rename ( char *, lv_t *);
 int    lv_setup_for_create ( char *, vg_t **, char *, int *,
                              uint, uint, uint, uint, uint, uint, char **);
@@ -406,7 +421,10 @@ int    lvm_unlock ( void);
 /* LVMTAB based functions */
 int  lvm_tab_read ( char **, int *);
 int  lvm_tab_write ( char *, int);
+int  lvm_tab_check_free_lv_numbers ( vg_t *);
+int  lvm_tab_check_free_vg_number ( vg_t *);
 int  lvm_tab_create ( void);
+int  *lvm_tab_get_all_vg_numbers ( void);
 int  lvm_tab_get_free_vg_number ( void);
 int  lvm_tab_lv_check_exist ( char *);
 int  lvm_tab_lv_read_by_name ( char *, char*, lv_t **);
@@ -792,4 +810,31 @@ static inline unsigned long get_pe_offset(ulong p, pv_t *pv)
 	return pv->pe_start + (p * pv->pe_size);
 }
 
+static inline ulong get_pe_from_offset(ulong offset, pv_t *pv)
+{
+	return (offset - pv->pe_start) / pv->pe_size;
+}
+
+static inline int lvm_pv_check_version(pv_t *pv)
+{
+	return (pv->version == 2 || pv->version == LVM_STRUCT_VERSION);
+}
+
+/* find next unused PV number */
+static inline int _find_unused_pv_number ( vg_t *vg) {
+   int p, u;
+   pv_t *pp;
+
+   for ( u = 1; u <= vg->pv_max; u++) {
+      for ( p = 0; p < vg->pv_max; p++) {
+         pp = vg->pv[p];
+         if ( pp == NULL) continue;
+         if ( pp->pv_number == u) break;
+      }
+      if ( p == vg->pv_max) return u;
+   }
+
+   return -1;
+}
+
 #endif /* #ifndef _LIBLVM_H_INCLUDE */
