@@ -1,7 +1,7 @@
 /*
  * tools/lib/lv_create_node.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * May 1997
  * January 2000
@@ -33,26 +33,29 @@
  *               - changed to new lv_create_kdev_t()
  *    27/07/1998 - enhanced error check
  *    31/01/2000 - use debug_enter()/debug_leave()
+ *    15/02/2001 - don't create device nodes if we're using devfs
  *
  */
 
 #include <liblvm.h>
-#include <grp.h>
-#include <sys/types.h>
 
 int lv_create_node ( lv_t *lv) {
+   gid_t gid = 0;
    int ret = 0;
-   struct group *group;
+   struct group *grent = NULL;
 
-#ifdef DEBUG
-   debug_enter ( "lv_create_node -- CALLED\n");
-#endif
+   debug_enter("lv_create_node -- CALLED with %s\n", lv ? lv->lv_name : "NULL");
 
    if ( lv == NULL || lv_check_consistency ( lv) < 0) {
       ret = -LVM_EPARAM;
       goto lv_create_node_end;
    }
 
+   if (lvm_check_devfs()) {
+       ret = 0;
+       goto lv_create_node_end;
+   }
+   
    if ( unlink ( lv->lv_name) == -1) {
       if ( errno != ENOENT) {
          ret = -LVM_ELV_CREATE_NODE_UNLINK;
@@ -60,16 +63,16 @@ int lv_create_node ( lv_t *lv) {
       }
    }
 
-   if (mknod(lv->lv_name, S_IFBLK | S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP,
-            lv->lv_dev) == -1)
-      ret = -LVM_ELV_CREATE_MODE_MKNOD;
-   else if (!(group = getgrnam("disk")) ||
-           chown(lv->lv_name, -1, group->gr_gid) == -1)
-      ret = -LVM_ELV_CREATE_MODE_CHMOD;
+   if ( ( grent = getgrnam ( "disk")) != NULL) gid = grent->gr_gid;
+   
+   if ( mknod ( lv->lv_name, S_IFBLK | S_IRUSR | S_IWUSR | S_IRGRP,
+                lv->lv_dev) == -1)
+      ret = -LVM_ELV_CREATE_NODE_MKNOD;
+   else if ( chmod ( lv->lv_name, 0660) == -1) ret = -LVM_ELV_CREATE_NODE_CHMOD;
+   else if ( chown ( lv->lv_name, 0, gid) == -1) ret = -LVM_ELV_CREATE_NODE_CHOWN;
 
 lv_create_node_end:
-#ifdef DEBUG
+
    debug_leave ( "lv_create_node -- LEAVING with %d\n", ret);
-#endif
    return ret;
 }
