@@ -62,15 +62,15 @@ static dir_cache_t *_dir_cache;
 static int _cache_size;
 static int _array_size;
 static char *_devdir[] = {
-	"/dev/ida",
-	"/dev/ide/hd",
-	"/dev/loop",
-	"/dev/md",
-	"/dev/rd",
-	"/dev/sd",
-	"/dev/cciss",
-	"/dev/ubd",
-	"/dev",
+	LVM_DIR_PREFIX "ida",
+	LVM_DIR_PREFIX "ide/hd",
+	LVM_DIR_PREFIX "loop",
+	LVM_DIR_PREFIX "md",
+	LVM_DIR_PREFIX "rd",
+	LVM_DIR_PREFIX "sd",
+	LVM_DIR_PREFIX "cciss",
+	LVM_DIR_PREFIX "ubd",
+	LVM_DIR_PREFIX "",
 	NULL
 };
 
