@@ -5,20 +5,21 @@
  *
  * February,March,July,October 2000
  *
- * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * LVM is distributed in the hope that it will be useful,
+ *
+ * This LVM library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This LVM library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
- * the Free Software Foundation, 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA. 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this LVM library; if not, write to the Free
+ * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+ * MA 02111-1307, USA
  *
  */
 
@@ -231,7 +232,7 @@ char *lvm_error ( int error) {
       { LVM_EPV_MOVE_PES_REALLOC, "pv_move_pes(): realloc"},
       { LVM_EPV_READ_ALL_PE_OF_VG_MALLOC, "pv_read_all_pe_of_vg(): malloc"},
       { LVM_EPV_READ_ALL_PE_OF_VG_PV_NUMBER,
-        "pv_read-all_pe_of_vg(): PV number"},
+        "pv_read_all_pe_of_vg(): PV number"},
       { LVM_EPV_READ_ALL_PV_LVM_DIR_CACHE, "pv_read_all_pv(): lvm_dir_cache"},
       { LVM_EPV_READ_ALL_PV_MALLOC, "pv_read_all_pv(): malloc"},
       { LVM_EPV_READ_ALL_PV_OF_VG_MALLOC, "pv_read_all_pv_of_vg(): malloc"},
