@@ -5,20 +5,21 @@
  *
  * March,July 2000
  *
- * LVM is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * LVM is distributed in the hope that it will be useful,
+ *
+ * This LVM library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This LVM library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
- * the Free Software Foundation, 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA. 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this LVM library; if not, write to the Free
+ * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+ * MA 02111-1307, USA
  *
  */
 
@@ -31,6 +32,8 @@
 
 #include <liblvm.h>
 
+/* if you change this string you will have to alter the lvm_check_uuid
+   fn. below */
 static unsigned char c[] = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
 unsigned char *lvm_create_uuid ( int len) {
@@ -38,9 +41,8 @@ unsigned char *lvm_create_uuid ( int len) {
    static unsigned char uuidstr[NAME_LEN+1];
    char *ret = NULL;
 
-#ifdef DEBUG
    debug_enter ( "lvm_create_uuid -- CALLED\n");
-#endif
+
    if ( len > 0) {
       if ( len > NAME_LEN) len = NAME_LEN;
       memset ( uuidstr, 0, sizeof ( uuidstr));
@@ -54,34 +56,31 @@ unsigned char *lvm_create_uuid ( int len) {
       }
    }
 
-#ifdef DEBUG
-   debug_leave ( "lvm_create_uuid -- LEAVING with uuidstr: \"%s\"\n", uuidstr);
-#endif
+   debug_leave ( "lvm_create_uuid -- LEAVING with uuidstr: \"%s\"\n",
+                 lvm_show_uuid ( uuidstr));
    return uuidstr;
 }
 
 
 int lvm_check_uuid ( char *uuidstr) {
-   int ret = 0, i, j, len;
+   int ret = -1;
+   char *ptr;
 
-#ifdef DEBUG
    debug_enter ( "lvm_check_uuid -- CALLED with uuidstr: \"%s\"\n", uuidstr);
-#endif
-   len = strlen ( uuidstr);
-   if ( uuidstr == NULL || len != UUID_LEN) ret = -1;
-   else {
-      for ( i = 0; i < len; i++) {
-         for ( j = 0; c[j] != 0; j++) {
-            if ( uuidstr[i] == c[j]) break;
-         }
-         if ( c[j] == 0) break;
-      }
-      if ( uuidstr[i] != 0) ret = -1;
-   }
 
-#ifdef DEBUG
+   if ( uuidstr == NULL || strlen ( uuidstr) != UUID_LEN)
+      goto out;
+
+   for(ptr = uuidstr; *ptr; ptr++)
+      if (!((*ptr >= '0' && *ptr <= '9') ||
+            (*ptr >= 'a' && *ptr <= 'z') ||
+            (*ptr >= 'A' && *ptr <= 'Z')))
+         goto out;
+   ret = 0;
+
+ out:
+
    debug_leave ( "lvm_check_uuid -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
 
