@@ -50,9 +50,8 @@ void   pv_show_free ( int, int);
 
 
 void pv_show_colon ( pv_t *pv) {
-#ifdef DEBUG
+
    debug_enter ( "pv_show_colon -- CALLED\n");
-#endif
 
    if ( pv != NULL) {
       printf ( "%s:%s:%d:%d:%d:%d:%d:%d:%d:%d:%d:%s\n",
@@ -71,9 +70,7 @@ void pv_show_colon ( pv_t *pv) {
              );
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_show_colon -- LEAVING\n");
-#endif
 }
 
 
@@ -81,9 +78,7 @@ void pv_show ( pv_t *pv) {
    ulong pe_free;
    char *dummy = NULL;
 
-#ifdef DEBUG
    debug_enter ( "pv_show -- CALLED\n");
-#endif
 
    if ( pv != NULL) {
       printf ( "---");
@@ -129,17 +124,14 @@ void pv_show ( pv_t *pv) {
                strlen(pv->pv_uuid) > 0 ? lvm_show_uuid ( pv->pv_uuid) : "none");
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_show -- LEAVING\n");
-#endif
    return;
 }
 
 
 void pv_show_short ( pv_t *pv) {
-#ifdef DEBUG
+
    debug_enter ( "pv_show_short -- CALLED\n");
-#endif
 
    if ( pv != NULL) {
       printf ( "PV Name (#)           %s (%u)\n", pv->pv_name, pv->pv_number);
@@ -152,9 +144,7 @@ void pv_show_short ( pv_t *pv) {
                pv->pe_total, pv->pe_total - pv->pe_allocated);
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_show_short -- LEAVING\n");
-#endif
    return;
 }
 
@@ -162,9 +152,7 @@ void pv_show_short ( pv_t *pv) {
 void pv_show_all_pv_of_vg ( vg_t *vg) {
    int p = 0;
 
-#ifdef DEBUG
    debug_enter ( "pv_show_all_pv_of_vg -- CALLED\n");
-#endif
 
    if ( vg != NULL &&
         vg_check_consistency ( vg) == 0) {
@@ -174,9 +162,7 @@ void pv_show_all_pv_of_vg ( vg_t *vg) {
       }
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_show_all_pv_of_vg -- LEAVING\n");
-#endif
    return;
 }
 
@@ -184,9 +170,7 @@ void pv_show_all_pv_of_vg ( vg_t *vg) {
 void pv_show_all_pv_of_vg_short ( vg_t *vg) {
    int p = 0;
 
-#ifdef DEBUG
    debug_enter ( "pv_show_all_pv_of_vg_short -- CALLED\n");
-#endif
 
    if ( vg != NULL &&
         vg_check_consistency ( vg) == 0) {
@@ -199,9 +183,7 @@ void pv_show_all_pv_of_vg_short ( vg_t *vg) {
       } else printf ( "NONE???\n");
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_show_all_pv_of_vg_short -- LEAVING\n");
-#endif
    return;
 }
 
@@ -209,9 +191,7 @@ void pv_show_all_pv_of_vg_short ( vg_t *vg) {
 void pv_show_pe ( pv_t *pv, pe_disk_t *pe, int pe_count) {
    int p;
 
-#ifdef DEBUG
    debug_enter ( "pv_show_pe -- CALLED\n");
-#endif
 
    if ( pv != NULL &&
         pv_check_consistency ( pv) == 0 &&
@@ -221,9 +201,7 @@ void pv_show_pe ( pv_t *pv, pe_disk_t *pe, int pe_count) {
                   p, pv->vg_name, pe[p].lv_num, pe[p].le_num);
    }
 
-#ifdef DEBUG
    debug_leave ( "pv_show_pe -- LEAVING\n");
-#endif
    return;
 }
 
@@ -243,9 +221,7 @@ int pv_show_pe_text ( pv_t *pv, pe_disk_t *pe, int pe_count) {
    pe_disk_t *pe_this = NULL;
    vg_t *vg = NULL;
 
-#ifdef DEBUG
    debug_enter ( "pv_show_text -- CALLED\n");
-#endif
 
    if ( pv == NULL || pe == NULL || pe_count == 0 ||
         pv_check_consistency ( pv) < 0) {
@@ -331,7 +307,7 @@ int pv_show_pe_text ( pv_t *pv, pe_disk_t *pe, int pe_count) {
          printf ( "   %05d %-25s ", p, lv_name_this);
          if ( strlen ( lv_name_this) > 25)
             printf ( "\n                                  ");
-         printf ( "%05d   %d\n", pe[p].le_num, LVM_PE_DISK_OFFSET( p, pv));
+         printf ( "%05d   %ld\n", pe[p].le_num, get_pe_offset(p, pv));
 
       } else if ( pe_free == -1) pe_free = p;
    }
@@ -344,22 +320,19 @@ pv_show_pe_text_end:
    if ( pe_this != NULL)            free ( pe_this);
    if ( pe_this_count != NULL)      free ( pe_this_count);
 
-#ifdef DEBUG
    debug_leave ( "pv_show_text -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 } /* pv_show_pe_text() */
 
 
 void  pv_show_free ( int pe_free, int p) {
-#ifdef DEBUG
+
    debug_enter ( "pv_show_free -- CALLED\n");
-#endif
+
    if ( p - pe_free > 1) printf ( "   %05d free\n   .....\n   %05d free\n",
                                   pe_free, p - 1);
    else                  printf ( "   %05d free\n", pe_free);
-#ifdef DEBUG
+
    debug_leave ( "pv_show_free -- LEAVING\n");
-#endif
    return;
 }
