@@ -1,7 +1,7 @@
 /*
  * tools/lvm_user.h
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * March 1997
  * May 1998
@@ -30,12 +30,10 @@
  *    16/05/1998 - added macro LVMTAB_CHECK
  *    01/01/1999 - ported to libc6
  *    01/22/1999 - added exit states of commands
+ *    15/09/1999 - enhanced check with lvm_check_kernel_lvmtab_consistency()
  *
  */
 
-char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
-                    "Mauelshagen  01/07/1999\n";
-
 #ifndef _LVM_USER_H_INCLUDE
 #define _LVM_USER_H_INCLUDE
 
@@ -45,7 +43,6 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #include <linux/genhd.h>
 #include <linux/major.h>
 
-#if ( __GLIBC__ > 1)
 #include <stdio.h>
 #include <fcntl.h>
 #include <malloc.h>
@@ -55,34 +52,24 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #include <ctype.h>
 #include <sys/stat.h>
 
+#include <getopt.h>
+#include <lvm.h>
+
+char *lvm_version = "Logical Volume Manager "LVM_RELEASE_NAME"\nHeinz Mauelshagen, Sistina Software  "LVM_RELEASE_DATE;
+
+#ifdef DEBUG
+#	define DEBUG_HELP           "\t[-d/--debug]\n"
+#	define DEBUG_HELP_2         "[-d|--debug] "
+#	define DEBUG_SHORT_OPTION   "d"
+#	define DEBUG_LONG_OPTION    { "debug", no_argument, NULL, 'd'},
+#	define DEBUG_HANDLE_CASE_D  case 'd': opt_d++; break
 #else
-
-#include <ctype.h>
-#include <fcntl.h>
-#include <malloc.h>
-#include <stdio.h>
-#include <string.h>
-#include <time.h>
-#include <unistd.h>
-#include <sys/ioctl.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,0,1)
-#  define _I386_STAT_H
+#	define DEBUG_HELP
+#	define DEBUG_HELP_2
+#	define DEBUG_SHORT_OPTION
+#	define DEBUG_LONG_OPTION
+#	define DEBUG_HANDLE_CASE_D
 #endif
-#include <sys/stat.h>
-#include <linux/genhd.h>
-#include <linux/major.h>
-#include <linux/fs.h>
-
-#endif /* if ( __GLIBC__ > 1) */
-
-#include <linux/lvm.h>
-
-#define	SUSER_CHECK { \
-   if ( ! ( getuid () == 0 && geteuid () == 0)) { \
-      fprintf ( stderr, "%s -- this command is for root only\n\n", cmd); \
-      return 1; \
-   } \
-}
 
 #define	LVMFILE_CHECK( chkfile) { \
    int file = -1; \
@@ -97,6 +84,12 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #define	LVMTAB_CHECK { \
    LVMFILE_CHECK ( LVMTAB); \
    LVMFILE_CHECK ( LVMTAB_DIR); \
+   if ( lvm_check_kernel_lvmtab_consistency () == FALSE) { \
+      fprintf ( stderr, "%s -- ERROR: VGDA in kernel and lvmtab are NOT " \
+                        "consistent; please run vgscan\n\n", \
+                        cmd); \
+      return LVM_ELVMTAB; \
+   } \
 }
 
 #define LVM_CHECK_IOP { \
@@ -106,7 +99,8 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
       return LVM_EDRIVER; \
    } \
    if ( ret != LVM_LIB_IOP_VERSION) { \
-      fprintf ( stderr, "%s -- invalid i/o protocol version\n\n", cmd); \
+      fprintf ( stderr, "%s -- invalid i/o protocol version %d\n\n", \
+			cmd, ret); \
       return LVM_EINVALID_IOP; \
    } \
 }
@@ -118,7 +112,7 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
          fprintf ( stderr, "%s -- no such device while locking logical " \
                            "volume manager\n", cmd); \
       else if ( ret == -LVM_ELVM_LOCK_YET_LOCKED) { \
-         fprintf ( stderr, "%s -- logical volume manager yet locked\n", cmd); \
+         fprintf(stderr, "%s -- logical volume manager already locked\n", cmd);\
          return LVM_ELOCK; \
       } else \
          fprintf ( stderr, "%s -- ERROR %d locking logical volume manager\n", \
@@ -143,6 +137,64 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 }
 
 
+#define	CMD_CHECK_OPT_A_SET { \
+   if ( opt_A_set == 0) { \
+        char *lvm_autobackup = getenv ( "LVM_AUTOBACKUP"); \
+        if ( lvm_autobackup != NULL) { \
+           char *ptr = lvm_autobackup; \
+           while ( *ptr != 0) { *ptr = tolower ( *ptr); ptr++;} \
+           printf ( "%s -- INFO: using environment variable LVM_AUTOBACKUP" \
+                    " to set option A\n", cmd); \
+           if ( strcmp ( lvm_autobackup, "no") == 0) opt_A = 0; \
+           else if ( strcmp ( lvm_autobackup, "yes") == 0) opt_A = 1; \
+           else { \
+              fprintf ( stderr, "%s -- ERROR: environment variable " \
+                                "LVM_AUTOBACKUP has invalid value " \
+                                "\"%s\"!\n\n", \
+                                cmd, lvm_autobackup); \
+              return LVM_EINVALID_CMD_LINE; \
+           } \
+        } \
+   } \
+}
+
+#define LVM_CHECK_DEFAULT_VG_NAME( lv_name, buffer, len) { \
+{ \
+   if ( strchr ( lv_name, '/') == NULL) { \
+      char *lvm_default_vg_name = getenv ( "LVM_VG_NAME"); \
+      if ( lvm_default_vg_name != NULL) { \
+         if ( strlen ( lv_name) < len - strlen ( lvm_default_vg_name) - sizeof ( LVM_DIR_PREFIX)) { \
+            if ( strchr ( lvm_default_vg_name, '/') == NULL) { \
+               sprintf ( buffer, "%s%s/%s%c", LVM_DIR_PREFIX, lvm_default_vg_name, lv_name, 0); \
+            } else { \
+               sprintf ( buffer, "%s/%s%c", lvm_default_vg_name, lv_name, 0); \
+            } \
+            lv_name = buffer; \
+         } \
+      } \
+      else lv_name = NULL; \
+   } \
+} \
+}
+
+
+#define LVM_GET_DEFAULT_VG_NAME( vg_name, buffer, len) { \
+{ \
+   char *lvm_default_vg_name = getenv ( "LVM_VG_NAME"); \
+   if ( lvm_default_vg_name != NULL) { \
+      if ( len > strlen ( lvm_default_vg_name) + sizeof ( LVM_DIR_PREFIX)) { \
+         if ( strchr ( lvm_default_vg_name, '/') == NULL) { \
+            sprintf ( buffer, "%s%s%c", LVM_DIR_PREFIX, lvm_default_vg_name, 0); \
+         } else { \
+            strcpy ( buffer, lvm_default_vg_name); \
+         } \
+         vg_name = buffer; \
+      } else vg_name = NULL; \
+   } \
+} \
+}
+
+
 /* return codes of the tools */
 #define	LVM_EDRIVER				95
 #define	LVM_EINVALID_IOP			96
@@ -185,85 +237,94 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 
 
 /* lvcreate */
-#define	LVM_ELVCREATE_VG_NAME			1
-#define	LVM_ELVCREATE_VG_CHECK_EXIST		2
-#define	LVM_ELVCREATE_VG_CHECK_ACTIVE		3
-#define	LVM_ELVCREATE_LV_CHECK_NAME		4
-#define	LVM_ELVCREATE_LV_LSTAT			5
-#define	LVM_ELVCREATE_LV_CHECK_EXIST		6
-#define	LVM_ELVCREATE_PV_CHECK_NAME		7
-#define	LVM_ELVCREATE_PV_NUMBER			8
-#define	LVM_ELVCREATE_STRIPES			9
-#define	LVM_ELVCREATE_STRIPE_SIZE		10
-#define	LVM_ELVCREATE_VG_STATUS			11
-#define	LVM_ELVCREATE_LV_SIZE			12
-#define	LVM_ELVCREATE_PE_FREE			13
-#define	LVM_ELVCREATE_STRIPE_COUNT		14
-#define	LVM_ELVCREATE_VG_READ			15
-#define	LVM_ELVCREATE_PV_CHECK_IN_VG		16
-#define	LVM_ELVCREATE_PV_READ			17
-#define	LVM_ELVCREATE_LV_COUNT			18
-#define	LVM_ELVCREATE_VG_SIZE			19
-#define	LVM_ELVCREATE_LV_SETUP			20
-#define	LVM_ELVCREATE_LV_CREATE			21
-#define	LVM_ELVCREATE_VG_WRITE			22
-#define	LVM_ELVCREATE_LV_CREATE_NODE		23
-#define	LVM_ELVCREATE_LV_OPEN			24
-#define	LVM_ELVCREATE_LV_WRITE			25
-#define LVM_ELVCREATE_READ_AHEAD		26
-#define LVM_ELVCREATE_NO_DEV			27
+#define	LVM_ELVCREATE_VG_NAME				1
+#define	LVM_ELVCREATE_VG_CHECK_EXIST			2
+#define	LVM_ELVCREATE_VG_CHECK_ACTIVE			3
+#define	LVM_ELVCREATE_LV_CHECK_NAME			4
+#define	LVM_ELVCREATE_LV_LSTAT				5
+#define	LVM_ELVCREATE_LV_CHECK_EXIST			6
+#define	LVM_ELVCREATE_PV_CHECK_NAME			7
+#define	LVM_ELVCREATE_PV_NUMBER				8
+#define	LVM_ELVCREATE_STRIPES				9
+#define	LVM_ELVCREATE_STRIPE_SIZE			10
+#define	LVM_ELVCREATE_VG_STATUS				11
+#define	LVM_ELVCREATE_LV_SIZE				12
+#define	LVM_ELVCREATE_PE_FREE				13
+#define	LVM_ELVCREATE_STRIPE_COUNT			14
+#define	LVM_ELVCREATE_VG_READ				15
+#define	LVM_ELVCREATE_PV_CHECK_IN_VG			16
+#define	LVM_ELVCREATE_PV_READ				17
+#define	LVM_ELVCREATE_LV_COUNT				18
+#define	LVM_ELVCREATE_VG_SIZE				19
+#define	LVM_ELVCREATE_LV_SETUP				20
+#define	LVM_ELVCREATE_LV_CREATE				21
+#define	LVM_ELVCREATE_VG_WRITE				22
+#define	LVM_ELVCREATE_LV_CREATE_NODE			23
+#define	LVM_ELVCREATE_LV_OPEN				24
+#define	LVM_ELVCREATE_LV_WRITE				25
+#define LVM_ELVCREATE_READ_AHEAD			26
+#define LVM_ELVCREATE_NO_DEV				27
+#define LVM_ELVCREATE_LV_NAME				28
+#define LVM_ELVCREATE_LV_SETUP_COW_TABLE_FOR_CREATE    	29
+#define LVM_ELVCREATE_LV_INIT_COW_TABLE			30
+#define LVM_ELVCREATE_LV_STATUS_BYNAME			31
+#define LVM_ELVCREATE_LV_SNAPSHOT_EXISTS		32
 
 /* lvdisplay */
-#define	LVM_ELVDISPLAY_LV_MISSING		1
+#define	LVM_ELVDISPLAY_LV_MISSING			1
 
 /* lvextend */
-#define	LVM_ELVEXTEND_LV_MISSING		1
-#define	LVM_ELVEXTEND_PV_NAME			2
-#define	LVM_ELVEXTEND_LV_NAME			3
-#define	LVM_ELVEXTEND_LV_CHECK_EXIST		4
-#define	LVM_ELVEXTEND_LV_CHECK_ACTIVE		5
-#define	LVM_ELVEXTEND_VG_READ			6
-#define	LVM_ELVEXTEND_PV_CHECK_IN_VG		7
-#define	LVM_ELVEXTEND_LV_GET_INDEX		8
-#define	LVM_ELVEXTEND_LV_SIZE			9
-#define	LVM_ELVEXTEND_LV_SIZE_FREE		10
-#define	LVM_ELVEXTEND_LV_SIZE_MAX		11
-#define	LVM_ELVEXTEND_LV_SETUP			12
-#define	LVM_ELVEXTEND_LV_EXTEND			13
-#define	LVM_ELVEXTEND_VG_WRITE			14
-#define	LVM_ELVEXTEND_LV_GET_INDEX_BY_NAME	15
+#define	LVM_ELVEXTEND_LV_MISSING			1
+#define	LVM_ELVEXTEND_PV_NAME				2
+#define	LVM_ELVEXTEND_LV_NAME				3
+#define	LVM_ELVEXTEND_LV_CHECK_EXIST			4
+#define	LVM_ELVEXTEND_LV_CHECK_ACTIVE			5
+#define	LVM_ELVEXTEND_VG_READ				6
+#define	LVM_ELVEXTEND_PV_CHECK_IN_VG			7
+#define	LVM_ELVEXTEND_LV_GET_INDEX			8
+#define	LVM_ELVEXTEND_LV_SIZE				9
+#define	LVM_ELVEXTEND_LV_SIZE_FREE			10
+#define	LVM_ELVEXTEND_LV_SIZE_MAX			11
+#define	LVM_ELVEXTEND_LV_SETUP				12
+#define	LVM_ELVEXTEND_LV_EXTEND				13
+#define	LVM_ELVEXTEND_VG_WRITE				14
+#define	LVM_ELVEXTEND_LV_GET_INDEX_BY_NAME		15
+#define	LVM_ELVEXTEND_LV_STATUS_BYNAME			16
+#define LVM_ELVEXTEND_LV_SETUP_COW_TABLE_FOR_CREATE    	17
 
 /* lvmchange */
-#define	LVM_ELVMCHANGE_OPEN			1
-#define	LVM_ELVMCHANGE_RESET			2
-#define	LVM_ELVMCHANGE_READ_AHEAD		3
+#define	LVM_ELVMCHANGE_OPEN				1
+#define	LVM_ELVMCHANGE_RESET				2
+#define	LVM_ELVMCHANGE_READ_AHEAD			3
 
 /* lvmdiskscan */
-#define	LVM_ELVMDISKSCAN_NO_FILES_FOUND		1
-#define	LVM_ELVMDISKSCAN_NO_DISKS_FOUND		2
+#define	LVM_ELVMDISKSCAN_NO_FILES_FOUND			1
+#define	LVM_ELVMDISKSCAN_NO_DISKS_FOUND			2
 
 /* lvmsadc */
-#define	LVM_ELVMSADC_NO_LOG_FILE		1
-#define	LVM_ELVMSADC_FOPEN			2
-#define	LVM_ELVMSADC_NO_VGS			3
-#define	LVM_ELVMSADC_FCLOSE			4
+#define	LVM_ELVMSADC_NO_LOG_FILE			1
+#define	LVM_ELVMSADC_FOPEN				2
+#define	LVM_ELVMSADC_NO_VGS				3
+#define	LVM_ELVMSADC_FCLOSE				4
 
 /* lvmsar */
-#define	LVM_ELVMSAR_FOPEN			1
-#define	LVM_ELVMSAR_FCLOSE			2
+#define	LVM_ELVMSAR_FOPEN				1
+#define	LVM_ELVMSAR_FCLOSE				2
 
 /* lvreduce */
-#define	LVM_ELVREDUCE_LV_MISSING		1
-#define	LVM_ELVREDUCE_LV_NAME			2
-#define	LVM_ELVREDUCE_LV_CHECK_ACTIVE		3
-#define	LVM_ELVREDUCE_LV_CHECK_EXIST		4
-#define	LVM_ELVREDUCE_VG_READ			5
-#define	LVM_ELVREDUCE_LV_GET_INDEX		6
-#define	LVM_ELVREDUCE_LV_SIZE			7
-#define	LVM_ELVREDUCE_LV_SETUP			8
-#define	LVM_ELVREDUCE_LV_REDUCE			9
-#define	LVM_ELVREDUCE_VG_WRITE			10
-#define	LVM_ELVREDUCE_LV_GET_INDEX_BY_NAME	11
+#define	LVM_ELVREDUCE_LV_MISSING			1
+#define	LVM_ELVREDUCE_LV_NAME				2
+#define	LVM_ELVREDUCE_LV_CHECK_ACTIVE			3
+#define	LVM_ELVREDUCE_LV_CHECK_EXIST			4
+#define	LVM_ELVREDUCE_VG_READ				5
+#define	LVM_ELVREDUCE_LV_GET_INDEX			6
+#define	LVM_ELVREDUCE_LV_SIZE				7
+#define	LVM_ELVREDUCE_LV_SETUP				8
+#define	LVM_ELVREDUCE_LV_REDUCE				9
+#define	LVM_ELVREDUCE_VG_WRITE				10
+#define	LVM_ELVREDUCE_LV_GET_INDEX_BY_NAME		11
+#define	LVM_ELVREDUCE_LV_STATUS_BYNAME			12
+#define LVM_ELVREDUCE_LV_SETUP_COW_TABLE_FOR_CREATE    	17
 
 /* lvremove */
 #define	LVM_ELVREMOVE_LV_MISSING		1
@@ -277,6 +338,7 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #define	LVM_ELVREMOVE_LV_RELEASE		9
 #define	LVM_ELVREMOVE_LV_REMOVE			10
 #define	LVM_ELVREMOVE_VG_WRITE			11
+#define	LVM_ELVREMOVE_LV_SNAPSHOT		12
 
 /* lvrename */
 #define LVM_ELVRENAME_LV_NAME			1
@@ -286,13 +348,13 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #define LVM_ELVRENAME_LV_CHECK_EXIST_OLD	5
 #define LVM_ELVRENAME_VG_NAME_DIFFER		6
 #define LVM_ELVRENAME_LV_CHECK_EXIST_NEW	7
-#define LVM_ELVRENAME_LV_CHECK_ACTIVE		8
 #define LVM_ELVRENAME_VG_READ			9
 #define LVM_ELVRENAME_LV_GET_INDEX		10
-#define LVM_ELVRENAME_LV_REMOVE_OLD		11
+#define LVM_ELVRENAME_LV_RENAME			11
 #define LVM_ELVRENAME_LV_CREATE			12
 #define LVM_ELVRENAME_VG_WRITE			13
 #define LVM_ELVRENAME_LV_CREATE_NODE		14
+#define LVM_ELVRENAME_VG_NAME			15
 
 /* lvscan */
 #define LVM_ELVSCAN_NO_VGS			1
@@ -310,6 +372,9 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #define	LVM_EPVCREATE_VG_CHECK_EXIST		2
 #define	LVM_EPVCREATE_PV_SETUP			3
 #define	LVM_EPVCREATE_PV_WRITE			4
+#define	LVM_EPVCREATE_INVALID_ID		5
+#define	LVM_EPVCREATE_PV_CHECK_NAME		6
+#define	LVM_EPVCREATE_PV_GET_SIZE		7
 
 /* pvdata */
 #define	LVM_EPVDATA_PV_MISSING			1
@@ -330,7 +395,6 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #define	LVM_EPVMOVE_VG_CHECK_CONSISTENCY	8
 #define	LVM_EPVMOVE_PV_GET_INDEX		9
 #define	LVM_EPVMOVE_PE_ALLOCATED		10
-#define	LVM_EPVMOVE_PV_CUR			11
 #define	LVM_EPVMOVE_LV_CHECK_NAME		12
 #define	LVM_EPVMOVE_LV_NUMBER			13
 #define	LVM_EPVMOVE_LV_CHECK_ON_PV		14
@@ -367,14 +431,16 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #define	LVM_EVGCFGRESTORE_VG_CFGBACKUP		13
 
 /* vgchange */
-#define	LVM_EVGCHANGE_VG_WRITE			1
-#define	LVM_EVGCHANGE_VG_EXTEND			2
-#define	LVM_EVGCHANGE_VG_CFG_BACKUP		3
-#define	LVM_EVGCHANGE_PE_OVERLAP		4
-#define	LVM_EVGCHANGE_VG_CFGBACKUP		5
-#define	LVM_EVGCHANGE_VG_CFGBACKUP_LVMTAB	6
+#define	LVM_EVGCHANGE_VG_WRITE				1
+#define	LVM_EVGCHANGE_VG_EXTEND				2
+#define	LVM_EVGCHANGE_VG_CFG_BACKUP			3
+#define	LVM_EVGCHANGE_PE_OVERLAP			4
+#define	LVM_EVGCHANGE_VG_CFGBACKUP			5
+#define	LVM_EVGCHANGE_VG_CFGBACKUP_LVMTAB		6
 #define	LVM_EVGCHANGE_VG_CREATE_DIR_AND_GROUP_NODES	7
 #define	LVM_EVGCHANGE_VG_REMOVE_DIR_AND_GROUP_NODES	8
+#define	LVM_EVGCHANGE_LV_SETUP_COW_TABLE_FOR_CREATE	9
+#define	LVM_EVGCHANGE_LV_READ_COW_TABLE			10
 
 /* vgck */
 #define	LVM_EVGCK_CHECK_ERRORS			1
@@ -400,6 +466,7 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #define	LVM_EVGCREATE_VG_INSERT			18
 #define	LVM_EVGCREATE_VG_CFGBACKUP		19
 #define	LVM_EVGCREATE_VG_CFGBACKUP_LVMTAB	20
+#define	LVM_EVGCREATE_VG_CHECK_DIR		21
 
 /* vgdisplay */
 #define	LVM_EVGDISPLAY_VG_READ			1
@@ -407,6 +474,7 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #define	LVM_EVGDISPLAY_PV_COUNT			3
 #define	LVM_EVGDISPLAY_VG_NOT_FOUND		4
 #define	LVM_EVGDISPLAY_NO_VG			5
+#define LVM_EVGDISPLAY_VG_CFGRESTORE		6
 
 /* vgexport */
 #define	LVM_EVGEXPORT_NO_VG			1
@@ -489,24 +557,30 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #define LVM_EVGREMOVE_PV_SETUP			2
 #define LVM_EVGREMOVE_PV_WRITE_ALL_PV_OF_VG	3
 #define LVM_EVGREMOVE_VG_REMOVE			4
+#define LVM_EVGREMOVE_LV_EXISTS			5
+#define LVM_EVGREMOVE_VG_CHECK_NAME		6
+#define LVM_EVGREMOVE_VG_CHECK_ACTIVE		7
+#define LVM_EVGREMOVE_PV_ONLINE			8
+#define LVM_EVGREMOVE_VG_ERROR			9
+#define LVM_EVGREMOVE_VG_READ			10
 
 /* vgrename */
-#define LVM_EVGRENAME_VG_DIR_NEW		1
-#define LVM_EVGRENAME_VG_DIR_OLD		2
-#define LVM_EVGRENAME_VG_CHECK_NAME_NEW		3
-#define LVM_EVGRENAME_VG_CHECK_NAME_OLD		4
-#define LVM_EVGRENAME_VG_CHECK_EXIST_OLD	5
-#define LVM_EVGRENAME_VG_CHECK_ACTIVE		6
-#define LVM_EVGRENAME_VG_NAMES_IDENTICAL	7
-#define LVM_EVGRENAME_VG_CHECK_EXIST_NEW	8
-#define LVM_EVGRENAME_VG_READ			9
-#define LVM_EVGRENAME_VG_REMOVE			10
-#define LVM_EVGRENAME_LVM_TAB_VG_REMOVE		11
+#define LVM_EVGRENAME_VG_DIR_NEW			1
+#define LVM_EVGRENAME_VG_DIR_OLD			2
+#define LVM_EVGRENAME_VG_CHECK_NAME_NEW			3
+#define LVM_EVGRENAME_VG_CHECK_NAME_OLD			4
+#define LVM_EVGRENAME_VG_CHECK_EXIST_OLD		5
+#define LVM_EVGRENAME_VG_NAMES_IDENTICAL		7
+#define LVM_EVGRENAME_VG_CHECK_EXIST_NEW		8
+#define LVM_EVGRENAME_VG_READ				9
+#define LVM_EVGRENAME_VG_REMOVE				10
+#define LVM_EVGRENAME_LVM_TAB_VG_REMOVE			11
 #define LVM_EVGRENAME_VG_REMOVE_DIR_AND_GROUP_NODES	12
 #define LVM_EVGRENAME_VG_CREATE_DIR_AND_GROUP_NODES	13
-#define LVM_EVGRENAME_VG_INSERT			14
-#define LVM_EVGRENAME_VG_CFGBACKUP		15
-#define LVM_EVGRENAME_VG_CFGBACKUP_LVMTAB	16
+#define LVM_EVGRENAME_VG_INSERT				14
+#define LVM_EVGRENAME_VG_CFGBACKUP			15
+#define LVM_EVGRENAME_VG_CFGBACKUP_LVMTAB		16
+#define LVM_EVGRENAME_VG_RENAME				17
 
 /* vgscan */
 #define LVM_EVGSCAN_PV_READ_ALL_PV		1
@@ -515,6 +589,7 @@ char *lvm_version = "Logical Volume Manager 0.7 by Heinz "
 #define LVM_EVGSCAN_LVMTAB			4
 #define LVM_EVGSCAN_NO_DEV			5
 #define LVM_EVGSCAN_VG_WRITE			6
+#define LVM_EVGSCAN_DEV_REUSE			7
 
 /* vgsplit */
 #define LVM_EVGSPLIT_VG_CHECK_NAME		1
