@@ -1,12 +1,13 @@
 /*
  * tools/vgchange.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * March-May,October 1997
  * May,June,September 1998
  * February,July,October,December 1999
- * February 2000
+ * February,July 2000
+ * June 2001
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +43,18 @@
  *                 pv_create_name_from_kdev_t() wasn't able to get a name,
  *                 because /proc isn't mounted
  *    15/02/2000 - use lvm_error()
+ *    05/07/2000 - added reading copy on write exception table of snapshot
+ *                 logical volumes using new function lv_read_COW_table()
+ *    23/01/2001 - added call to lvm_init (JT)
+ *    05/03/2001 - fixed call to lv_setup_COW_table_for_create() in order to
+ *                 make use of the actual COW chunk size on disk rather than
+ *                 the DEFAULT one
+ *               - added check of the first array element after call to
+ *                 lv_read_COW_table in order to print information about
+ *                 the fact that the snapshot was dropped before VG deactivation
+ *               - replaced vg->lv[l] by lv_ptr
+ *    09/04/2001 - Tidy usage message and command-line options.
+ *    22/06/2001 - added Andreas Dilger's PE on 4k boundary alignment enhancements
  *
  */
 
@@ -53,6 +66,30 @@ char *cmd = NULL;
 int opt_d = 0;
 #endif
 
+void usage ( int ret)
+{
+    FILE *stream = stderr;
+
+    if (ret == 0) {
+	stream = stdout;
+	printf ( "%s (IOP %d)\n\n", lvm_version, LVM_LIB_IOP_VERSION);
+	printf ( "%s -- change volume group settings\n", cmd);
+    }
+
+    fprintf ( stream, "\n%s "
+	      "[-A|--autobackup {y|n}] "
+              DEBUG_HELP_2
+	      "[-h|--help]\n\t"
+	      "{-a|--available {y|n} [VolumeGroupName...] |\n\t "
+	      " -x|--allocation {y|n} [VolumeGroupName...]\n\t"
+	      " -l|--logicalvolume MaxLogicalVolumes}\n\t"
+	      "[-v|--verbose] "
+	      "[--version]\n",
+	      cmd);
+    exit ( ret);
+}
+
+
 int main ( int argc, char **argv) {
    int backup = FALSE;
    int c = 0;
@@ -65,21 +102,16 @@ int main ( int argc, char **argv) {
    int v = 0;
    char *dir = NULL;
    char *vg_name = NULL;
-#ifdef DEBUG
-   char *options = "A:a:dh?l:vx:";
-#else
-   char *options = "A:a:h?l:vx:";
-#endif
+   char *options = "A:a:h?l:vx:" DEBUG_SHORT_OPTION;
    struct option long_options[] = {
       { "autobackup",    required_argument, NULL, 'A'},
       { "available",     required_argument, NULL, 'a'},
-#ifdef DEBUG
-      { "debug",         no_argument,       NULL, 'd'},
-#endif
+      DEBUG_LONG_OPTION
       { "help",          no_argument,       NULL, 'h'},
       { "logicalvolume", required_argument, NULL, 'l'},
       { "verbose",       no_argument,       NULL, 'v'},
       { "allocation",    required_argument, NULL, 'x'},
+      { "version",       no_argument,       NULL, 22 },
       { NULL, 0, NULL, 0}
    };
    char **vg_name_ptr = NULL;
@@ -94,12 +126,13 @@ int main ( int argc, char **argv) {
    uint32_t pe_on_disk_base_sav;
    pv_t **pv_offl = NULL;
    pv_t **pv_incons = NULL;
+   lv_t *lv_ptr = NULL;
    vg_t *vg = NULL;
    vg_t *vg_s = NULL;
 
+   /* lvm_init(argc, argv); */
    cmd = basename ( argv[0]);
 
-   SUSER_CHECK;
    LVMTAB_CHECK;
 
    while ( ( c = getopt_long ( argc, argv, options,
@@ -139,34 +172,12 @@ int main ( int argc, char **argv) {
             return LVM_EINVALID_CMD_LINE;
             break;
 
-#ifdef DEBUG
-         case 'd':
-            if ( opt_d > 0) {
-               fprintf ( stderr, "%s -- d option already given\n\n", cmd);
-               return LVM_EINVALID_CMD_LINE;
-            }
-            opt_d++;
-            break;
-#endif
+         DEBUG_HANDLE_CASE_D;
 
          case 'h':
          case '?':
-            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Change\n\n"
-                     "Synopsis:\n"
-                     "---------\n\n"
-                     "%s\n"
-                     "\t[-A/--autobackup y/n]\n"
-                     "\t[-a/--available y/n [VolumeGroupName...]]\n"
-#ifdef DEBUG
-                     "\t[-d/--debug]\n"
-#endif
-                     "\t[-h/-?/--help]\n"
-                     "\t[-l/--logicalvolume MaxLogicalVolumes]\n"
-                     "\t[-v/--verbose]\n"
-                     "\t[-x/--allocation y/n [VolumeGroupName...]]\n\n",
-                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
-            return 0;
-            break;
+	     usage(0);
+	     break;
 
          case 'l':
             if ( opt_l > 0) {
@@ -214,10 +225,14 @@ int main ( int argc, char **argv) {
             opt_v++;
             break;
 
+         case 22:
+            printf ("%s: %s (IOP %d)\n", cmd, lvm_version, LVM_LIB_IOP_VERSION);
+            return 0;
+
          default:
             fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                       cmd, c);
-            return LVM_EINVALID_CMD_LINE;
+            usage ( LVM_EINVALID_CMD_LINE );
       }
    }
   
@@ -229,7 +244,7 @@ int main ( int argc, char **argv) {
       fprintf ( stderr, "%s -- you have to give the -a, -l or the -x "
                         "option\n\n",
                         cmd);
-      return LVM_EINVALID_CMD_LINE;
+      usage (LVM_EINVALID_CMD_LINE);
    }
    
    if ( opt_a + opt_l + opt_x > 1 ) {
@@ -240,7 +255,7 @@ int main ( int argc, char **argv) {
    }
    
    if ( opt_a == 1 && opt_A > 1) {
-      fprintf ( stderr, "%s -- -A option not neccessary with "
+      fprintf ( stderr, "%s -- -A option not necessary with "
                         "-a option\n\n", cmd);
       return LVM_EINVALID_CMD_LINE;
    }
@@ -301,7 +316,6 @@ int main ( int argc, char **argv) {
          else
             fprintf ( stderr, "%s -- volume group \"%s\" does not exist\n",
                               cmd, vg_name);
-         vg_deactivate ( vg_name);
          continue;
       }
 
@@ -322,7 +336,6 @@ int main ( int argc, char **argv) {
                               "volume group \"%s\"\n",
                               cmd, lvm_error ( ret), vg_name);
          }
-         if ( vg_check_active ( vg_name) == TRUE) vg_deactivate ( vg_name);
          continue;
       }
    
@@ -332,26 +345,25 @@ int main ( int argc, char **argv) {
          fprintf ( stderr, "%s -- ERROR \"%s\" volume group \"%s\" "
                            " is inconsistent\n",
                            cmd, lvm_error ( ret), vg_name);
-         vg_deactivate ( vg_name);
          continue;
       }
 
       lvm_dont_interrupt ( 0);
    
       if ( opt_a > 0) {
-         if ( opt_a_action == VG_CREATE) {
+         if ( opt_a_action & VG_CREATE) {
             if ( opt_v > 0) printf ( "%s -- checking if all physical volumes "
                                      "of volume group \"%s\" are available\n",
                                      cmd, vg_name);
-            if ((ret = vg_check_online_all_pv(vg, &pv_offl, &pv_incons)) < 0 &&
-		ret != -LVM_EPV_READ_MD_DEVICE) {
+            if ( ( ret = vg_check_online_all_pv ( vg, &pv_offl, &pv_incons)) < 0
+                 &&  ret != -LVM_EPV_READ_MD_DEVICE) {
                if ( ret == -LVM_EPV_READ_PV_CREATE_NAME_FROM_KDEV_T) {
                      fprintf ( stderr, "%s -- ERROR: can't get name(s) of "
                                        "physical volumes\n"
                                        "%s -- Please check, if /proc "
                                        "is mounted\n",
                                        cmd, cmd);
-               } else if ( ret != -LVM_EPV_READ_MD_DEVICE) {
+               } else {
                   if ( pv_offl != NULL) for ( p = 0; pv_offl[p] != NULL; p++) {
                      fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" of "
                                        "volume group \"%s\" is offline\n",
@@ -369,21 +381,45 @@ int main ( int argc, char **argv) {
                continue;
             }
       
-            /* Avoid snapshot logical volumes during activation */
+            /* check for snapshot logical volumes and get the COW table data */
             for ( l = 0; l < vg->lv_max; l++) {
-               if ( vg->lv[l] != NULL) {
-                  vg->lv[l]->lv_block_exception = NULL;
-                  vg->lv[l]->lv_remap_ptr = 0;
-                  vg->lv[l]->lv_remap_end = 0;
-                  if ( vg->lv[l]->lv_access & LV_SNAPSHOT_ORG) {
-                     printf ( "%s -- resetting source snapshot logical "
-                              "volume \"%s\"\n", cmd, vg->lv[l]->lv_name);
-                     vg->lv[l]->lv_access &= ~LV_SNAPSHOT_ORG;
-                  }
-                  if ( vg->lv[l]->lv_access & LV_SNAPSHOT) {
-                     printf ( "%s -- resetting destination snapshot logical "
-                              "volume \"%s\"\n", cmd, vg->lv[l]->lv_name);
-                     vg->lv[l]->lv_access &= ~LV_SNAPSHOT;
+               if ( ( lv_ptr = vg->lv[l]) != NULL) {
+                  lv_ptr->lv_block_exception = NULL;
+                  lv_ptr->lv_remap_ptr = 0;
+                  lv_ptr->lv_remap_end = 0;
+                  lv_ptr->lv_snapshot_prev = NULL;
+                  lv_ptr->lv_snapshot_next = NULL;
+
+                  if ( lv_ptr->lv_access & LV_SNAPSHOT_ORG) lv_ptr->lv_snapshot_org = lv_ptr;
+
+                  if ( lv_ptr->lv_access & LV_SNAPSHOT) {
+                     if ( ( ret = lv_setup_COW_table_for_create ( vg, vg->lv[lv_get_index_by_minor(vg, lv_ptr->lv_snapshot_minor)]->lv_name, l, lv_ptr->lv_chunk_size / 2)) < 0) {
+                        fprintf ( stderr, "%s -- ERROR \"%s\" setting up snapshot "
+                                          "copy on write exception table for \"%s\"\n\n",
+                                          cmd, lvm_error ( ret), lv_ptr->lv_name);
+                        return LVM_EVGCHANGE_LV_SETUP_COW_TABLE_FOR_CREATE;
+                     }
+                     if ( ( ret = lv_read_COW_table ( vg, lv_ptr)) < 0) {
+                           fprintf ( stderr, "%s -- ERROR %d reading copy on "
+                                             "write exception table of \"%s\"\n",
+                                             cmd, ret, lv_ptr->lv_name);
+                           return LVM_EVGCHANGE_LV_READ_COW_TABLE;
+                     }
+                     if ( lv_ptr->lv_block_exception[0].rsector_org ==
+                          LVM_SNAPSHOT_DROPPED_SECTOR) {
+                        lv_ptr->lv_status &= ~LV_ACTIVE;
+                        /* Needs to be 0 to avoid that the driver tries
+                           to hash link crap */
+                        lv_ptr->lv_remap_ptr = 0;
+                        printf ( "%s -- INFO: logical volume \"%s\" is a "
+                                 "dropped snapshot.\n"
+                                 "%s -- You can't use it any longer,"
+                                 " remove it please\n",
+                                 cmd, lv_ptr->lv_name,cmd);
+                     }
+                  } else {
+                     lv_ptr->lv_snapshot_minor = 0;
+                     lv_ptr->lv_chunk_size = 0;
                   }
                }
             }
@@ -391,19 +427,16 @@ int main ( int argc, char **argv) {
             if ( opt_v > 0) printf ( "%s -- creating VGDA for \"%s\" "
                                      "in kernel\n",
                                      cmd, vg_name);
-
-            ret = vg_create ( vg_name, vg);
-            if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN) {
-               vg_create_dir_and_group( vg);
-               ret = vg_create ( vg_name, vg);
+            if ( ( ret = vg_create ( vg_name, vg)) == -LVM_EVG_CREATE_REMOVE_OPEN) {
+                  vg_create_dir_and_group ( vg);
+                  ret = vg_create ( vg_name, vg);
             }
+
             if ( ret == 0)
                printf ( "%s -- volume group \"%s\" successfully activated\n",
                         cmd, vg_name);
             else {
-#ifdef DEBUG
                debug ( "%s -- vg_create returned: %d\n", cmd, ret);
-#endif
                if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN) {
                   fprintf ( stderr, "%s -- can't open logical volume manager "
                                     "to activate volume group \"%s\"\n",
@@ -431,12 +464,9 @@ int main ( int argc, char **argv) {
                printf ( "%s -- volume group \"%s\" successfully deactivated\n",
                         cmd, vg_name);
             else {
-#ifdef DEBUG
                debug ( "%s -- vg_remove returned: %d\n", cmd, ret);
-#endif
                if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN) {
-                  fprintf ( stderr, "%s -- can't open logical volume manager "
-                                    "to deactivate \"%s\"?\n", cmd, vg_name);
+                  fprintf ( stderr, "%s -- volume group \"%s\" is not active\n", cmd, vg_name);
                } else if ( ret == -EPERM) {
                   if ( opt_v > 0)
                      printf ( "%s -- getting volume group status from "
@@ -540,6 +570,8 @@ int main ( int argc, char **argv) {
 
       /* change MaxLogicalVolume */
       if ( opt_l > 0) {
+         pv_t *pp;
+
          if ( opt_v > 0) printf ( "%s -- checking for volume group \"%s\" "
                                   "activity\n", cmd, vg_name);
          if ( vg_check_active ( vg_name) == TRUE) {
@@ -583,15 +615,16 @@ int main ( int argc, char **argv) {
          for ( ; l < max_lv; l++) vg->lv[l] = NULL;
 
          vg->lv_max = max_lv;
-         for ( p = 0; p < vg->pv_max; p++) {
-            if ( vg->pv[p] != NULL) {
-               vg->pv[p]->lv_on_disk.size = ( max_lv + 1) * sizeof ( lv_disk_t);
-               pe_on_disk_base_sav = vg->pv[p]->pe_on_disk.base;
-               LVM_PE_ON_DISK_BASE ( vg->pv[p]);
-               vg->pv[p]->pe_on_disk.size -= vg->pv[p]->pe_on_disk.base -
-                                             pe_on_disk_base_sav;
-               if ( LVM_DISK_SIZE ( vg->pv[p]) / SECTOR_SIZE >
-                    vg->pv[p]->pv_size - vg->pv[p]->pe_total * vg->pe_size) {
+         for ( p = 0, pp = *vg->pv; p < vg->pv_max; p++, pp++) {
+            if ( pp != NULL) {
+		    pp->lv_on_disk.size =
+			    round_up((max_lv + 1) * sizeof ( lv_disk_t),
+				     LVM_VGDA_ALIGN);
+               pe_on_disk_base_sav = pp->pe_on_disk.base;
+               pp->pe_on_disk.base = pp->lv_on_disk.base + pp->lv_on_disk.size;
+               pp->pe_on_disk.size -= pp->pe_on_disk.base - pe_on_disk_base_sav;
+               if ( LVM_VGDA_SIZE ( pp) / SECTOR_SIZE >
+                    (pp->pv_size & ~LVM_PE_ALIGN) - pp->pe_total * pp->pe_size){
                   fprintf ( stderr, "%s -- extended VGDA would overlap "
                                     "first physical extent\n\n",
                                     cmd);
